/* original parser id follows */
/* yysccsid[] = "@(#)yaccpar	1.9 (Berkeley) 02/21/93" */
/* (use YYMAJOR/YYMINOR for ifdefs dependent on parser version) */

#define YYBYACC 1
#define YYMAJOR 2
#define YYMINOR 0
#define YYPATCH 20210109

#define YYEMPTY        (-1)
#define yyclearin      (yychar = YYEMPTY)
#define yyerrok        (yyerrflag = 0)
#define YYRECOVERING() (yyerrflag != 0)
#define YYENOMEM       (-2)
#define YYEOF          0
#define YYPREFIX "yy"

#define YYPURE 0

#line 33 "grammar.y"

#include "config.h"

#include <stdio.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <stdint.h>
#include <netinet/in.h>
#include <string.h>
#include <stdlib.h>
#include <errno.h>
#include <ctype.h>

#include "util.h"
#include "output_util.h"
#include "rbtree.h"
#include "filter.h"
#include "nfdump.h"
#include "nffile.h"
#include "nftree.h"
#include "ipconv.h"
#include "sgregex/sgregex.h"

#define AnyMask 0xffffffffffffffffLL

/*
 * function prototypes
 */
static void  yyerror(char *msg);

static uint32_t ChainHosts(uint64_t *offsets, uint64_t *hostlist, int num_records, int type);

static uint64_t VerifyMac(char *s);

static int InitSymbols(void);

static uint32_t Get_fwd_status_id(char *status);

static char *stripWord(char *word);

static int IsMD5(char *string);

enum { DIR_UNSPEC = 1, 
	   SOURCE, DESTINATION, SOURCE_AND_DESTINATION, SOURCE_OR_DESTINATION, 
	   DIR_IN, DIR_OUT, 
	   IN_SRC, IN_DST, OUT_SRC, OUT_DST, 
	   ADJ_PREV, ADJ_NEXT };

enum { IS_START = 0, IS_END };

/* var defs */
extern int 			lineno;
extern char 		*yytext;
extern uint64_t		*IPstack;
extern uint32_t	StartNode;
extern uint8_t	geoFilter;
extern uint8_t	ja3Filter;
extern int (*FilterEngine)(uint32_t *);
extern char	*FilterFilename;

static uint32_t num_ip;

static struct fwd_status_def_s {
	uint32_t	id;
	char		*name;
} fwd_status_def_list[] = {
	{ 0,	"Ukwn"}, 	/* Unknown*/
	{ 1,	"Forw"}, 	/* Normal forwarding*/
	{ 2,	"Frag"}, 	/* Fragmented*/
	{ 16,	"Drop"}, 	/* Drop*/
	{ 17,	"DaclD"},	/* Drop ACL deny*/
	{ 18,	"Daclp"},	/* Drop ACL drop*/
	{ 19,	"Noroute"},	/* Unroutable*/
	{ 20,	"Dadj"}, 	/* Drop Adjacency*/
	{ 21,	"Dfrag"}, 	/* Drop Fragmentation & DF set*/
	{ 22,	"Dbadh"}, 	/* Drop Bad header checksum*/
	{ 23,	"Dbadtlen"}, /* Drop Bad total Length*/
	{ 24,	"Dbadhlen"}, /* Drop Bad Header Length*/
	{ 25,	"DbadTTL"}, /* Drop bad TTL*/
	{ 26,	"Dpolicy"}, /* Drop Policer*/
	{ 27,	"Dwred"}, 	/* Drop WRED*/
	{ 28,	"Drpf"}, 	/* Drop RPF*/
	{ 29,	"Dforus"}, 	/* Drop For us*/
	{ 30,	"DbadOf"}, 	/* Drop Bad output interface*/
	{ 31,	"Dhw"}, 	/* Drop Hardware*/
	{ 128,	"Term"}, 	/* Terminate*/
	{ 129,	"Tadj"}, 	/* Terminate Punt Adjacency*/
	{ 130,	"TincAdj"}, /* Terminate Incomplete Adjacency*/
	{ 131,	"Tforus"}, 	/* Terminate For us*/
	{ 0,	NULL}		/* Last entry*/
};

static char **fwd_status = NULL;

char yyerror_buff[256];

#define MPLSMAX 0x00ffffff
#ifdef YYSTYPE
#undef  YYSTYPE_IS_DECLARED
#define YYSTYPE_IS_DECLARED 1
#endif
#ifndef YYSTYPE_IS_DECLARED
#define YYSTYPE_IS_DECLARED 1
#line 132 "grammar.y"
typedef union {
	uint64_t		value;
	char			*s;
	FilterParam_t	param;
	void			*list;
} YYSTYPE;
#endif /* !YYSTYPE_IS_DECLARED */
#line 133 "grammar.c"

/* compatibility with bison */
#ifdef YYPARSE_PARAM
/* compatibility with FreeBSD */
# ifdef YYPARSE_PARAM_TYPE
#  define YYPARSE_DECL() yyparse(YYPARSE_PARAM_TYPE YYPARSE_PARAM)
# else
#  define YYPARSE_DECL() yyparse(void *YYPARSE_PARAM)
# endif
#else
# define YYPARSE_DECL() yyparse(void)
#endif

/* Parameters sent to lex. */
#ifdef YYLEX_PARAM
# define YYLEX_DECL() yylex(void *YYLEX_PARAM)
# define YYLEX yylex(YYLEX_PARAM)
#else
# define YYLEX_DECL() yylex(void)
# define YYLEX yylex()
#endif

#if !(defined(yylex) || defined(YYSTATE))
int YYLEX_DECL();
#endif

/* Parameters sent to yyerror. */
#ifndef YYERROR_DECL
#define YYERROR_DECL() yyerror(const char *s)
#endif
#ifndef YYERROR_CALL
#define YYERROR_CALL(msg) yyerror(msg)
#endif

extern int YYPARSE_DECL();

#define ANY 257
#define IP 258
#define TUNIP 259
#define IF 260
#define MAC 261
#define MPLS 262
#define TOS 263
#define DIR 264
#define FLAGS 265
#define TUN 266
#define PROTO 267
#define MASK 268
#define NET 269
#define PORT 270
#define FWDSTAT 271
#define IN 272
#define OUT 273
#define SRC 274
#define DST 275
#define EQ 276
#define LT 277
#define GT 278
#define LE 279
#define GE 280
#define PREV 281
#define NEXT 282
#define IDENT 283
#define ENGINE_TYPE 284
#define ENGINE_ID 285
#define AS 286
#define GEO 287
#define PACKETS 288
#define BYTES 289
#define FLOWS 290
#define LABEL 291
#define NFVERSION 292
#define COUNT 293
#define PPS 294
#define BPS 295
#define BPP 296
#define DURATION 297
#define NOT 298
#define IPV4 299
#define IPV6 300
#define BGPNEXTHOP 301
#define ROUTER 302
#define VLAN 303
#define CLIENT 304
#define SERVER 305
#define APP 306
#define LATENCY 307
#define SYSID 308
#define ASA 309
#define DENIED 310
#define XEVENT 311
#define XNET 312
#define XPORT 313
#define INGRESS 314
#define EGRESS 315
#define ACL 316
#define ACE 317
#define XACE 318
#define NAT 319
#define ADD 320
#define EVENT 321
#define VRF 322
#define NPORT 323
#define NIP 324
#define PBLOCK 325
#define START 326
#define END 327
#define STEP 328
#define SIZE 329
#define PAYLOAD 330
#define CONTENT 331
#define REGEX 332
#define JA3 333
#define OBSERVATION 334
#define DOMAIN 335
#define POINT 336
#define ID 337
#define STRING 338
#define WORD 339
#define REASON 340
#define NUMBER 341
#define PORTNUM 342
#define ICMP_TYPE 343
#define ICMP_CODE 344
#define OR 345
#define AND 346
#define NEGATE 347
#define YYERRCODE 256
typedef short YYINT;
static const YYINT yylhs[] = {                           -1,
    0,    0,    3,    3,    3,    3,    3,    3,    3,    3,
    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,
    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,
    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,
    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,
    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,
    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,
    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,
    3,    3,    3,    7,    7,    7,    7,    7,    8,    8,
    8,    4,    4,    4,    4,    4,    4,    2,    2,    2,
    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
    2,    2,    6,    6,    5,    5,    5,    1,    1,    1,
    1,    1,    1,    1,
};
static const YYINT yylen[] = {                            2,
    0,    1,    1,    2,    1,    1,    2,    2,    3,    3,
    4,    4,    3,    3,    3,    3,    3,    3,    4,    3,
    3,    2,    2,    3,    3,    6,    3,    6,    3,    3,
    4,    4,    4,    2,    4,    4,    4,    4,    6,    2,
    2,    3,    3,    2,    3,    4,    4,    4,    4,    5,
    4,    4,    3,    4,    4,    4,    4,    4,    4,    3,
    3,    3,    3,    4,    3,    6,    4,    3,    6,    3,
    4,    5,    3,    3,    3,    4,    3,    2,    2,    2,
    2,    4,    4,    1,    3,    2,    3,    4,    1,    2,
    3,    0,    1,    1,    1,    1,    1,    0,    1,    1,
    3,    3,    3,    3,    1,    1,    2,    2,    2,    2,
    1,    1,    1,    1,    1,    1,    1,    1,    3,    3,
    2,    3,    5,    5,
};
static const YYINT yydefred[] = {                         0,
    3,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  111,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    5,    6,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  118,    0,    0,
    0,  113,  114,   80,   81,   93,   94,   95,   96,   97,
   22,   23,    0,    0,    8,    7,   79,   78,  107,  108,
  109,  110,    0,    0,    0,    0,    0,    4,    0,    0,
    0,   44,    0,    0,    0,    0,    0,    0,  121,    0,
    0,    0,    0,    0,   34,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,   40,
   41,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  115,  116,  117,    0,   77,    0,   21,   10,
    9,  101,  103,  102,  104,    0,   27,   42,   43,   13,
   20,   18,   14,   15,   16,   17,   29,   30,    0,    0,
    0,    0,    0,   45,    0,    0,    0,   53,    0,    0,
    0,    0,    0,   61,   62,    0,   65,    0,    0,    0,
    0,    0,  120,    0,   24,   25,   73,   75,    0,   70,
    0,    0,    0,    0,    0,   68,    0,    0,   74,    0,
    0,    0,    0,   60,    0,   76,    0,   31,   32,   33,
   49,   48,   47,   46,   36,   37,   54,   55,   56,   57,
   58,   64,   82,   83,    0,    0,    0,   19,   71,    0,
   38,    0,   35,    0,   67,   11,   12,    0,   51,   59,
    0,   52,    0,    0,  123,  124,    0,   72,   89,    0,
    0,   50,    0,    0,    0,   28,    0,   26,   90,   39,
    0,   69,   66,   85,    0,   87,   91,   88,
};
static const YYINT yydgoto[] = {                         45,
   46,   47,   48,   63,  136,   49,  244,  250,
};
static const YYINT yysindex[] = {                        70,
    0, -243, -291, -239, -246, -275, -254, -210, -205, -248,
 -245,    0, -215, -293, -149, -149, -149, -280, -149, -149,
 -149, -149, -149, -149,   70,    0,    0, -169, -164, -172,
 -167, -166, -199, -296, -178, -177, -175, -273, -240, -224,
 -198, -194,   70, -190,    0, -209,  -57,    0, -212, -189,
 -149,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, -176, -253,    0,    0,    0,    0,    0,    0,
    0,    0, -113, -112, -110, -108, -256,    0, -174, -173,
 -171,    0, -168, -165, -162, -159, -156, -155,    0, -160,
 -148, -149, -149, -149,    0, -149, -250, -149, -149, -244,
 -149, -149, -149, -149, -200, -150, -147, -145, -143,    0,
    0,  -37,  132,   70,   70, -255, -142, -146, -141, -149,
 -136, -140, -163, -154, -139, -149, -149, -134, -138, -149,
 -149, -252,    0,    0,    0, -149,    0, -133,    0,    0,
    0,    0,    0,    0,    0,   83,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, -132, -131,
 -127, -126, -263,    0, -125, -124, -123,    0, -122, -121,
 -120, -119, -118,    0,    0, -114,    0, -116, -115,  138,
   70, -111,    0,  136,    0,    0,    0,    0, -107,    0,
  -40,  -82, -105,  137, -104,    0, -103, -102,    0,  186,
 -101, -100,  151,    0,  -97,    0,  -93,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  -91,  -35,  -93,    0,    0,  -92,
    0,  -90,    0,  -90,    0,    0,    0,  -89,    0,    0,
  -93,    0,  201,  -36,    0,    0,  -32,    0,    0,  -44,
  -34,    0,  -26,  -88,  203,    0,  -84,    0,    0,    0,
  -83,    0,    0,    0,  -77,    0,    0,    0,
};
static const YYINT yyrindex[] = {                         2,
    0,    0,    0,  -73,    0,    0,    0,  133,  165,  204,
  236,    0,  306,    0,  -73,  -73,  -73,    0,  -73,  -73,
  -73,  -73,  -73,  -73,  275,    0,    0,    0,    0,    0,
    0,    0,    0,    0, -197, -184,    0,    0,    0,    0,
    0,    0,  275,    0,    0,  257,    0,    0,    0,    0,
  -73,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  -73,  -73,  -73,    0,  -73,  -73,  -73,  -73,  -73,
  -73,  -73,  -73,  -73,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  275,  275,    0,    0,    0,    0,  -73,
    0,    0,  -73,  -73,    0,  -73,  -73,    0,    0,  -73,
  -73,    0,    0,    0,    0,  -73,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    1,    0,    0,    0,    3,
  275,    5,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  -25,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  -22,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,
};
static const YYINT yygindex[] = {                         0,
  -12,    0,    0,   57,    0,    8, -179,   25,
};
#define YYTABLESIZE 630
static const YYINT yytable[] = {                        261,
   63,    1,  122,  180,  119,  246,  230,  257,  232,  261,
   55,  257,   89,   50,   96,  146,  184,  257,   84,  203,
   64,   86,   52,   53,   97,   56,   57,   58,   59,   60,
  112,   56,   57,   58,   59,   60,   56,   57,   58,   59,
   60,   63,   77,  122,   78,  119,   61,  247,  260,   54,
   52,   53,  101,  102,  103,  104,  256,   82,  262,  163,
  258,  253,   65,   69,   70,   66,  263,   84,   71,   72,
   86,   79,   80,   81,  212,   83,   84,   85,   86,   87,
   88,  147,  185,   67,  140,  204,   68,  141,   90,  164,
  105,  106,  107,   91,   51,  168,   73,   74,   62,   75,
   76,  182,  183,  133,  134,  135,   44,  138,  192,   43,
  108,  109,   56,   57,   58,   59,   60,  194,  113,  113,
  113,   56,   57,   58,   59,   60,   56,   57,   58,   59,
   60,  114,  114,  114,   92,  114,  115,  174,  175,   93,
   94,   95,  110,   98,   99,  100,  111,  113,  159,  160,
  161,  137,  162,  165,  166,  167,  169,  170,  171,  172,
  173,  142,  143,  144,  139,  145,  148,  149,  226,  150,
  213,  181,  151,  207,  225,  152,  189,  157,  153,  193,
  195,  154,  197,  198,  155,  156,  201,  202,  176,  158,
  177,  178,  205,  179,  187,  186,  188,  191,  196,  200,
  116,  117,  118,  119,  190,  120,  199,  206,  208,  209,
  121,  122,  123,  210,  211,  214,  215,  216,  217,  218,
  219,  220,  221,  222,  223,  224,  227,  234,  124,  125,
  126,  127,  238,  228,  115,  233,  235,  236,  237,  239,
  240,  241,  231,  242,  243,  128,  245,  254,  248,  265,
  249,  252,  264,  266,  129,  130,    2,  267,  251,   98,
   98,   98,   98,  268,   98,  131,  132,   92,    0,   98,
   98,   98,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,   98,   98,   98,
   98,    0,    0,    0,    0,    0,  259,  229,    0,    0,
    0,  255,    0,    0,   98,  255,  259,  114,  115,  114,
  115,  255,   84,   98,   98,   86,    0,    0,    0,    0,
    0,    0,    0,    0,   98,   98,    1,    0,    0,    0,
    0,    2,    0,    3,    4,    5,    6,    0,    0,    0,
    7,    8,    9,   10,   11,   63,   63,  122,  122,  119,
   12,   13,   14,   15,   16,    0,    0,    0,    0,   17,
   18,   19,   20,   21,   22,   23,   24,   25,   26,   27,
   28,   29,    0,   30,   31,   32,    0,   33,   34,    0,
    0,    0,    0,   35,   36,    0,    0,    0,   37,    0,
  105,  105,  105,  105,   38,  105,    0,    0,    0,   39,
  105,  105,  105,   40,    0,    0,    0,    0,    0,    0,
    0,    0,   41,   42,    0,    0,    0,    0,  105,  105,
  105,  105,  106,  106,  106,  106,    0,  106,    0,    0,
    0,    0,  106,  106,  106,  105,    0,    0,    0,    0,
    0,    0,    0,    0,  105,  105,    0,    0,    0,    0,
  106,  106,  106,  106,    0,  105,  105,    0,    0,    0,
    0,   99,   99,   99,   99,    0,   99,  106,    0,    0,
    0,   99,   99,   99,    0,    0,  106,  106,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  106,  106,   99,
   99,   99,   99,  100,  100,  100,  100,    0,  100,    0,
    0,    0,    0,  100,  100,  100,   99,    0,    0,    0,
    0,    0,    0,    0,    0,   99,   99,    0,    0,    0,
    0,  100,  100,  100,  100,    0,   99,   99,    0,    0,
    0,    0,   98,   98,   98,   98,    0,   98,  100,    0,
    0,    0,   98,   98,   98,    0,    0,  100,  100,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  100,  100,
   98,   98,   98,   98,  112,  112,  112,    0,  112,    0,
    0,    0,    0,  112,  112,  112,    0,   98,    0,    0,
    0,    0,    0,    0,    0,    0,   98,   98,    0,    0,
    0,  112,  112,  112,  112,    0,    0,   98,   98,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  112,    0,
    0,    0,    0,    0,    0,    0,    0,  112,  112,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  112,  112,
};
static const YYINT yycheck[] = {                         44,
    0,    0,    0,   41,    0,   41,   47,   44,   91,   44,
    3,   44,   25,  257,  311,  272,  272,   44,   44,  272,
  267,   44,  314,  315,  321,  276,  277,  278,  279,  280,
   43,  276,  277,  278,  279,  280,  276,  277,  278,  279,
  280,   41,  258,   41,  338,   41,  286,  227,   93,  341,
  314,  315,  326,  327,  328,  329,   93,  338,   93,  310,
   93,  241,  338,  274,  275,  341,   93,   93,  274,  275,
   93,   15,   16,   17,  338,   19,   20,   21,   22,   23,
   24,  338,  338,  338,  338,  338,  341,  341,  258,  340,
  331,  332,  333,  258,  338,  340,  345,  346,  338,  345,
  346,  114,  115,  316,  317,  318,   37,   51,  272,   40,
  335,  336,  276,  277,  278,  279,  280,  272,  316,  317,
  318,  276,  277,  278,  279,  280,  276,  277,  278,  279,
  280,  316,  317,  318,  307,  345,  346,  338,  339,  307,
  307,  341,  341,  322,  322,  321,  341,  338,   92,   93,
   94,  341,   96,   97,   98,   99,  100,  101,  102,  103,
  104,  275,  275,  274,  341,  274,  341,  341,  181,  341,
  163,   40,  341,   91,   37,  341,  120,  338,  341,  123,
  124,  341,  126,  127,  341,  341,  130,  131,  339,  338,
  338,  337,  136,  337,  341,  338,  338,  338,  338,  338,
  258,  259,  260,  261,  341,  263,  341,  341,  341,  341,
  268,  269,  270,  341,  341,  341,  341,  341,  341,  341,
  341,  341,  341,  338,  341,  341,   91,   91,  286,  287,
  288,  289,   47,  341,  346,  341,  341,  341,  341,  341,
  341,   91,  325,  341,  338,  303,  338,   47,  341,   47,
  341,  341,  341,  338,  312,  313,    0,  341,  234,  258,
  259,  260,  261,  341,  263,  323,  324,  341,   -1,  268,
  269,  270,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  286,  287,  288,
  289,   -1,   -1,   -1,   -1,   -1,  341,  338,   -1,   -1,
   -1,  338,   -1,   -1,  303,  338,  341,  345,  346,  345,
  346,  338,  338,  312,  313,  338,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  323,  324,  257,   -1,   -1,   -1,
   -1,  262,   -1,  264,  265,  266,  267,   -1,   -1,   -1,
  271,  272,  273,  274,  275,  345,  346,  345,  346,  345,
  281,  282,  283,  284,  285,   -1,   -1,   -1,   -1,  290,
  291,  292,  293,  294,  295,  296,  297,  298,  299,  300,
  301,  302,   -1,  304,  305,  306,   -1,  308,  309,   -1,
   -1,   -1,   -1,  314,  315,   -1,   -1,   -1,  319,   -1,
  258,  259,  260,  261,  325,  263,   -1,   -1,   -1,  330,
  268,  269,  270,  334,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  343,  344,   -1,   -1,   -1,   -1,  286,  287,
  288,  289,  258,  259,  260,  261,   -1,  263,   -1,   -1,
   -1,   -1,  268,  269,  270,  303,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  312,  313,   -1,   -1,   -1,   -1,
  286,  287,  288,  289,   -1,  323,  324,   -1,   -1,   -1,
   -1,  258,  259,  260,  261,   -1,  263,  303,   -1,   -1,
   -1,  268,  269,  270,   -1,   -1,  312,  313,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  323,  324,  286,
  287,  288,  289,  258,  259,  260,  261,   -1,  263,   -1,
   -1,   -1,   -1,  268,  269,  270,  303,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  312,  313,   -1,   -1,   -1,
   -1,  286,  287,  288,  289,   -1,  323,  324,   -1,   -1,
   -1,   -1,  258,  259,  260,  261,   -1,  263,  303,   -1,
   -1,   -1,  268,  269,  270,   -1,   -1,  312,  313,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  323,  324,
  286,  287,  288,  289,  259,  260,  261,   -1,  263,   -1,
   -1,   -1,   -1,  268,  269,  270,   -1,  303,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  312,  313,   -1,   -1,
   -1,  286,  287,  288,  289,   -1,   -1,  323,  324,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  303,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  312,  313,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  323,  324,
};
#define YYFINAL 45
#ifndef YYDEBUG
#define YYDEBUG 0
#endif
#define YYMAXTOKEN 347
#define YYUNDFTOKEN 358
#define YYTRANSLATE(a) ((a) > YYMAXTOKEN ? YYUNDFTOKEN : (a))
#if YYDEBUG
static const char *const yyname[] = {

"end-of-file",0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,"'%'",0,0,"'('","')'","'*'","'+'","','",0,0,"'/'",0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,"'['",0,"']'",0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,"ANY","IP","TUNIP","IF","MAC","MPLS","TOS","DIR","FLAGS","TUN","PROTO","MASK",
"NET","PORT","FWDSTAT","IN","OUT","SRC","DST","EQ","LT","GT","LE","GE","PREV",
"NEXT","IDENT","ENGINE_TYPE","ENGINE_ID","AS","GEO","PACKETS","BYTES","FLOWS",
"LABEL","NFVERSION","COUNT","PPS","BPS","BPP","DURATION","NOT","IPV4","IPV6",
"BGPNEXTHOP","ROUTER","VLAN","CLIENT","SERVER","APP","LATENCY","SYSID","ASA",
"DENIED","XEVENT","XNET","XPORT","INGRESS","EGRESS","ACL","ACE","XACE","NAT",
"ADD","EVENT","VRF","NPORT","NIP","PBLOCK","START","END","STEP","SIZE",
"PAYLOAD","CONTENT","REGEX","JA3","OBSERVATION","DOMAIN","POINT","ID","STRING",
"WORD","REASON","NUMBER","PORTNUM","ICMP_TYPE","ICMP_CODE","OR","AND","NEGATE",
0,0,0,0,0,0,0,0,0,0,"illegal-symbol",
};
static const char *const yyrule[] = {
"$accept : prog",
"prog :",
"prog : expr",
"term : ANY",
"term : IDENT STRING",
"term : IPV4",
"term : IPV6",
"term : PROTO NUMBER",
"term : PROTO STRING",
"term : TUN PROTO NUMBER",
"term : TUN PROTO STRING",
"term : dqual PACKETS comp NUMBER",
"term : dqual BYTES comp NUMBER",
"term : FLOWS comp NUMBER",
"term : PPS comp NUMBER",
"term : BPS comp NUMBER",
"term : BPP comp NUMBER",
"term : DURATION comp NUMBER",
"term : COUNT comp NUMBER",
"term : dqual TOS comp NUMBER",
"term : NFVERSION comp NUMBER",
"term : FLAGS comp NUMBER",
"term : FLAGS AS",
"term : FLAGS STRING",
"term : dqual IP STRING",
"term : dqual TUNIP STRING",
"term : dqual IP IN '[' iplist ']'",
"term : NEXT IP STRING",
"term : NEXT IP IN '[' iplist ']'",
"term : BGPNEXTHOP IP STRING",
"term : ROUTER IP STRING",
"term : CLIENT LATENCY comp NUMBER",
"term : SERVER LATENCY comp NUMBER",
"term : APP LATENCY comp NUMBER",
"term : SYSID NUMBER",
"term : dqual PORT comp NUMBER",
"term : INGRESS VRF comp NUMBER",
"term : EGRESS VRF comp NUMBER",
"term : dqual PORT IN PBLOCK",
"term : dqual PORT IN '[' ullist ']'",
"term : ICMP_TYPE NUMBER",
"term : ICMP_CODE NUMBER",
"term : ENGINE_TYPE comp NUMBER",
"term : ENGINE_ID comp NUMBER",
"term : LABEL STRING",
"term : ASA EVENT REASON",
"term : ASA EVENT comp NUMBER",
"term : ASA EVENT DENIED inout",
"term : ASA EVENT DENIED STRING",
"term : ASA XEVENT comp NUMBER",
"term : dqual XNET STRING '/' NUMBER",
"term : dqual XPORT comp NUMBER",
"term : inout acl comp NUMBER",
"term : NAT EVENT REASON",
"term : NAT EVENT comp NUMBER",
"term : PBLOCK START comp NUMBER",
"term : PBLOCK END comp NUMBER",
"term : PBLOCK STEP comp NUMBER",
"term : PBLOCK SIZE comp NUMBER",
"term : dqual NPORT comp NUMBER",
"term : dqual NIP STRING",
"term : PAYLOAD CONTENT STRING",
"term : PAYLOAD CONTENT WORD",
"term : PAYLOAD REGEX WORD",
"term : PAYLOAD REGEX WORD STRING",
"term : PAYLOAD JA3 STRING",
"term : dqual NIP IN '[' iplist ']'",
"term : dqual AS comp NUMBER",
"term : dqual GEO STRING",
"term : dqual AS IN '[' ullist ']'",
"term : dqual MASK NUMBER",
"term : dqual NET STRING STRING",
"term : dqual NET STRING '/' NUMBER",
"term : dqual IF NUMBER",
"term : dqual VLAN NUMBER",
"term : dqual MAC STRING",
"term : MPLS STRING comp NUMBER",
"term : MPLS ANY NUMBER",
"term : FWDSTAT NUMBER",
"term : FWDSTAT STRING",
"term : DIR NUMBER",
"term : DIR inout",
"term : OBSERVATION DOMAIN ID NUMBER",
"term : OBSERVATION POINT ID NUMBER",
"iplist : STRING",
"iplist : STRING '/' NUMBER",
"iplist : iplist STRING",
"iplist : iplist ',' STRING",
"iplist : iplist STRING '/' NUMBER",
"ullist : NUMBER",
"ullist : ullist NUMBER",
"ullist : ullist ',' NUMBER",
"comp :",
"comp : EQ",
"comp : LT",
"comp : GT",
"comp : LE",
"comp : GE",
"dqual :",
"dqual : SRC",
"dqual : DST",
"dqual : SRC OR DST",
"dqual : DST OR SRC",
"dqual : SRC AND DST",
"dqual : DST AND SRC",
"dqual : IN",
"dqual : OUT",
"dqual : IN SRC",
"dqual : IN DST",
"dqual : OUT SRC",
"dqual : OUT DST",
"dqual : PREV",
"dqual : NEXT",
"inout : INGRESS",
"inout : EGRESS",
"acl : ACL",
"acl : ACE",
"acl : XACE",
"expr : term",
"expr : expr OR expr",
"expr : expr AND expr",
"expr : NOT expr",
"expr : '(' expr ')'",
"expr : '(' expr ')' '%' STRING",
"expr : '%' STRING '(' expr ')'",

};
#endif

#if YYDEBUG
int      yydebug;
#endif

int      yyerrflag;
int      yychar;
YYSTYPE  yyval;
YYSTYPE  yylval;
int      yynerrs;

/* define the initial stack-sizes */
#ifdef YYSTACKSIZE
#undef YYMAXDEPTH
#define YYMAXDEPTH  YYSTACKSIZE
#else
#ifdef YYMAXDEPTH
#define YYSTACKSIZE YYMAXDEPTH
#else
#define YYSTACKSIZE 10000
#define YYMAXDEPTH  10000
#endif
#endif

#define YYINITSTACKSIZE 200

typedef struct {
    unsigned stacksize;
    YYINT    *s_base;
    YYINT    *s_mark;
    YYINT    *s_last;
    YYSTYPE  *l_base;
    YYSTYPE  *l_mark;
} YYSTACKDATA;
/* variables for the parser stack */
static YYSTACKDATA yystack;
#line 2392 "grammar.y"

static void  yyerror(char *msg) {

	if ( FilterFilename )
		snprintf(yyerror_buff, 255 ,"File '%s' line %d: %s at '%s'", FilterFilename, lineno, msg, yytext);
	else 
		snprintf(yyerror_buff, 255, "Line %d: %s at '%s'", lineno, msg, yytext);

	yyerror_buff[255] = '\0';
	fprintf(stderr, "%s\n", yyerror_buff);

} /* End of yyerror */

static uint32_t ChainHosts(uint64_t *offsets, uint64_t *hostlist, int num_records, int type) {
uint32_t offset_a, offset_b, i, j, block;
	if ( type == SOURCE ) {
		offset_a = offsets[0];
		offset_b = offsets[1];
	} else {
		offset_a = offsets[2];
		offset_b = offsets[3];
	}

	i = 0;
	block = Connect_AND(
				NewBlock(offset_b, MaskIPv6, hostlist[i+1] , CMP_EQ, FUNC_NONE, NULL ),
				NewBlock(offset_a, MaskIPv6, hostlist[i] , CMP_EQ, FUNC_NONE, NULL )
			);
	i += 2;
	for ( j=1; j<num_records; j++ ) {
		uint32_t b = Connect_AND(
				NewBlock(offset_b, MaskIPv6, hostlist[i+1] , CMP_EQ, FUNC_NONE, NULL ),
				NewBlock(offset_a, MaskIPv6, hostlist[i] , CMP_EQ, FUNC_NONE, NULL )
			);
		block = Connect_OR(block, b);
		i += 2;
	}

	return block;

} // End of ChainHosts

uint64_t VerifyMac(char *s) {
uint64_t mac;
size_t slen = strlen(s);
long l;
char *p, *q, *r;
int i;

	if ( slen > 17 )
		return 0; 

	for (i=0; i<slen; i++ ) {
		if ( !isxdigit(s[i]) && s[i] != ':' ) 
			return 0;
	}

	p = strdup(s);
	if ( !p ) {
		yyerror("malloc() error");
		return 0;
	}

	mac = 0;
	i = 0;	// number of MAC octets must be 6
	r = p;
	q = strchr(r, ':');
	while ( r && i < 6 ) {
		if ( q ) 
			*q = '\0';
		l = strtol(r, NULL, 16);
		if ( (i == 0 && errno == EINVAL) ) {
			free(p);
			return 0;
		}
		if ( l > 255 ) {
			free(p);
			return 0;
		}

		mac = ( mac << 8 ) | (l & 0xFF );
		i++;

		if ( q ) {
			r = ++q;
			q = strchr(r, ':');
		} else 
			r = NULL;
	}

	if ( i != 6 )
		return 0;

	return mac;

} // End of VerifyMac

static int InitSymbols(void) {
int i;

	// already initialised?
	if ( fwd_status )
		return 1;

	// fill fwd status cache table
	fwd_status = ( char **)calloc(256, sizeof(char *));
	if ( !fwd_status ) {
		fprintf(stderr, "malloc(): %s line %d: %s", __FILE__, __LINE__, strerror (errno));
		return 0;
	}
	i=0;
	while ( fwd_status_def_list[i].name ) {
		uint32_t j = fwd_status_def_list[i].id;
		fwd_status[j] = fwd_status_def_list[i].name;
		i++;
	}
	return 1;

} // End of InitSymbols

static uint32_t Get_fwd_status_id(char *status) {
int i;

	if ( !fwd_status && !InitSymbols() )
		yyerror("malloc() error");

	i = 0;
	while ( i < 256 ) {
		if ( fwd_status[i] && strcasecmp(fwd_status[i], status) == 0 ) 
			return i;
		i++;
	}
	return 256;

} // End of Get_fwd_status_id

static char *stripWord(char *word) {
	char *w = strdup(word);

	// strip ", ' and /' from begin/end of string
	if ( w[0] == 0x27 || w[0] == 0x22 || w[0] == 0x2f ) w++;
  size_t last = strlen(w) -1;

	if ( w[last] == 0x27 || w[last] == 0x22 || w[last] == 0x2f ) w[last] = '\0';

	return w;
} // End of stripWord

static int IsMD5(char *string) {

	int i = 0;
	for (i=0; i<32; i++) {
		char c = string[i];
		if ( c == '\0' || !isxdigit(c))
		return 0;
	}
	return string[i] == '\0';

} // End of IsMD5
#line 870 "grammar.c"

#if YYDEBUG
#include <stdio.h>	/* needed for printf */
#endif

#include <stdlib.h>	/* needed for malloc, etc */
#include <string.h>	/* needed for memset */

/* allocate initial stack or double stack size, up to YYMAXDEPTH */
static int yygrowstack(YYSTACKDATA *data)
{
    int i;
    unsigned newsize;
    YYINT *newss;
    YYSTYPE *newvs;

    if ((newsize = data->stacksize) == 0)
        newsize = YYINITSTACKSIZE;
    else if (newsize >= YYMAXDEPTH)
        return YYENOMEM;
    else if ((newsize *= 2) > YYMAXDEPTH)
        newsize = YYMAXDEPTH;

    i = (int) (data->s_mark - data->s_base);
    newss = (YYINT *)realloc(data->s_base, newsize * sizeof(*newss));
    if (newss == 0)
        return YYENOMEM;

    data->s_base = newss;
    data->s_mark = newss + i;

    newvs = (YYSTYPE *)realloc(data->l_base, newsize * sizeof(*newvs));
    if (newvs == 0)
        return YYENOMEM;

    data->l_base = newvs;
    data->l_mark = newvs + i;

    data->stacksize = newsize;
    data->s_last = data->s_base + newsize - 1;
    return 0;
}

#if YYPURE || defined(YY_NO_LEAKS)
static void yyfreestack(YYSTACKDATA *data)
{
    free(data->s_base);
    free(data->l_base);
    memset(data, 0, sizeof(*data));
}
#else
#define yyfreestack(data) /* nothing */
#endif

#define YYABORT  goto yyabort
#define YYREJECT goto yyabort
#define YYACCEPT goto yyaccept
#define YYERROR  goto yyerrlab

int
YYPARSE_DECL()
{
    int yym, yyn, yystate;
#if YYDEBUG
    const char *yys;

    if ((yys = getenv("YYDEBUG")) != 0)
    {
        yyn = *yys;
        if (yyn >= '0' && yyn <= '9')
            yydebug = yyn - '0';
    }
#endif

    yym = 0;
    yyn = 0;
    yynerrs = 0;
    yyerrflag = 0;
    yychar = YYEMPTY;
    yystate = 0;

#if YYPURE
    memset(&yystack, 0, sizeof(yystack));
#endif

    if (yystack.s_base == NULL && yygrowstack(&yystack) == YYENOMEM) goto yyoverflow;
    yystack.s_mark = yystack.s_base;
    yystack.l_mark = yystack.l_base;
    yystate = 0;
    *yystack.s_mark = 0;

yyloop:
    if ((yyn = yydefred[yystate]) != 0) goto yyreduce;
    if (yychar < 0)
    {
        yychar = YYLEX;
        if (yychar < 0) yychar = YYEOF;
#if YYDEBUG
        if (yydebug)
        {
            if ((yys = yyname[YYTRANSLATE(yychar)]) == NULL) yys = yyname[YYUNDFTOKEN];
            printf("%sdebug: state %d, reading %d (%s)\n",
                    YYPREFIX, yystate, yychar, yys);
        }
#endif
    }
    if (((yyn = yysindex[yystate]) != 0) && (yyn += yychar) >= 0 &&
            yyn <= YYTABLESIZE && yycheck[yyn] == (YYINT) yychar)
    {
#if YYDEBUG
        if (yydebug)
            printf("%sdebug: state %d, shifting to state %d\n",
                    YYPREFIX, yystate, yytable[yyn]);
#endif
        if (yystack.s_mark >= yystack.s_last && yygrowstack(&yystack) == YYENOMEM) goto yyoverflow;
        yystate = yytable[yyn];
        *++yystack.s_mark = yytable[yyn];
        *++yystack.l_mark = yylval;
        yychar = YYEMPTY;
        if (yyerrflag > 0)  --yyerrflag;
        goto yyloop;
    }
    if (((yyn = yyrindex[yystate]) != 0) && (yyn += yychar) >= 0 &&
            yyn <= YYTABLESIZE && yycheck[yyn] == (YYINT) yychar)
    {
        yyn = yytable[yyn];
        goto yyreduce;
    }
    if (yyerrflag != 0) goto yyinrecovery;

    YYERROR_CALL("syntax error");

    goto yyerrlab; /* redundant goto avoids 'unused label' warning */
yyerrlab:
    ++yynerrs;

yyinrecovery:
    if (yyerrflag < 3)
    {
        yyerrflag = 3;
        for (;;)
        {
            if (((yyn = yysindex[*yystack.s_mark]) != 0) && (yyn += YYERRCODE) >= 0 &&
                    yyn <= YYTABLESIZE && yycheck[yyn] == (YYINT) YYERRCODE)
            {
#if YYDEBUG
                if (yydebug)
                    printf("%sdebug: state %d, error recovery shifting\
 to state %d\n", YYPREFIX, *yystack.s_mark, yytable[yyn]);
#endif
                if (yystack.s_mark >= yystack.s_last && yygrowstack(&yystack) == YYENOMEM) goto yyoverflow;
                yystate = yytable[yyn];
                *++yystack.s_mark = yytable[yyn];
                *++yystack.l_mark = yylval;
                goto yyloop;
            }
            else
            {
#if YYDEBUG
                if (yydebug)
                    printf("%sdebug: error recovery discarding state %d\n",
                            YYPREFIX, *yystack.s_mark);
#endif
                if (yystack.s_mark <= yystack.s_base) goto yyabort;
                --yystack.s_mark;
                --yystack.l_mark;
            }
        }
    }
    else
    {
        if (yychar == YYEOF) goto yyabort;
#if YYDEBUG
        if (yydebug)
        {
            if ((yys = yyname[YYTRANSLATE(yychar)]) == NULL) yys = yyname[YYUNDFTOKEN];
            printf("%sdebug: state %d, error recovery discards token %d (%s)\n",
                    YYPREFIX, yystate, yychar, yys);
        }
#endif
        yychar = YYEMPTY;
        goto yyloop;
    }

yyreduce:
#if YYDEBUG
    if (yydebug)
        printf("%sdebug: state %d, reducing by rule %d (%s)\n",
                YYPREFIX, yystate, yyn, yyrule[yyn]);
#endif
    yym = yylen[yyn];
    if (yym > 0)
        yyval = yystack.l_mark[1-yym];
    else
        memset(&yyval, 0, sizeof yyval);

    switch (yyn)
    {
case 2:
#line 161 "grammar.y"
	{   
		StartNode = yystack.l_mark[0].value; 
	}
break;
case 3:
#line 166 "grammar.y"
	{ /* this is an unconditionally true expression, as a filter applies in any case */
		yyval.param.self = NewBlock(OffsetProto, 0, 0, CMP_EQ, FUNC_NONE, (void *)-1 ); 
	}
break;
case 4:
#line 170 "grammar.y"
	{	
		if ( !ScreenIdentString(yystack.l_mark[0].s) ) {
			yyerror("Illegal ident string");
			YYABORT;
		}

		uint32_t	index = AddIdent(yystack.l_mark[0].s);
		yyval.param.self = NewBlock(0, 0, index, CMP_IDENT, FUNC_NONE, NULL ); 
	}
break;
case 5:
#line 180 "grammar.y"
	{ 
		yyval.param.self = NewBlock(OffsetRecordMFlags, (1LL << ShiftRecordMFlags)  & MaskRecordMFlags, 
					(0LL << ShiftRecordMFlags)  & MaskRecordMFlags, CMP_EQ, FUNC_NONE, NULL); 
	}
break;
case 6:
#line 185 "grammar.y"
	{ 
		yyval.param.self = NewBlock(OffsetRecordMFlags, (1LL << ShiftRecordMFlags)  & MaskRecordMFlags, 
					(1LL << ShiftRecordMFlags)  & MaskRecordMFlags, CMP_EQ, FUNC_NONE, NULL); 
	}
break;
case 7:
#line 190 "grammar.y"
	{ 
		int64_t	proto;
		proto = yystack.l_mark[0].value;

		if ( proto > 255 ) {
			yyerror("Protocol number > 255");
			YYABORT;
		}
		if ( proto < 0 ) {
			yyerror("Unknown protocol");
			YYABORT;
		}
		yyval.param.self = NewBlock(OffsetProto, MaskProto, (proto << ShiftProto)  & MaskProto, CMP_EQ, FUNC_NONE, NULL); 

	}
break;
case 8:
#line 206 "grammar.y"
	{ 
		int64_t	proto;
		proto = ProtoNum(yystack.l_mark[0].s);

		if ( proto > 255 ) {
			yyerror("Protocol number > 255");
			YYABORT;
		}
		if ( proto < 0 ) {
			yyerror("Unknown protocol");
			YYABORT;
		}
		yyval.param.self = NewBlock(OffsetProto, MaskProto, (proto << ShiftProto)  & MaskProto, CMP_EQ, FUNC_NONE, NULL); 
	}
break;
case 9:
#line 221 "grammar.y"
	{ 
		int64_t	proto;
		proto = yystack.l_mark[0].value;

		if ( proto > 255 ) {
			yyerror("Tunnel protocol number > 255");
			YYABORT;
		}
		if ( proto < 0 ) {
			yyerror("Unknown tunnel protocol");
			YYABORT;
		}
		yyval.param.self = NewBlock(OffsetTUNPROTO, MaskTUNPROTO, (proto << ShiftTUNPROTO)  & MaskTUNPROTO, CMP_EQ, FUNC_NONE, NULL); 

	}
break;
case 10:
#line 237 "grammar.y"
	{ 
		int64_t	proto;
		proto = ProtoNum(yystack.l_mark[0].s);

		if ( proto > 255 ) {
			yyerror("Tunnel protocol number > 255");
			YYABORT;
		}
		if ( proto < 0 ) {
			yyerror("Unknown tunnel protocol");
			YYABORT;
		}
		yyval.param.self = NewBlock(OffsetTUNPROTO, MaskTUNPROTO, (proto << ShiftTUNPROTO)  & MaskTUNPROTO, CMP_EQ, FUNC_NONE, NULL); 
	}
break;
case 11:
#line 253 "grammar.y"
	{ 

		switch ( yystack.l_mark[-3].param.direction ) {
			case DIR_UNSPEC:
			case DIR_IN: 
				yyval.param.self = NewBlock(OffsetPackets, MaskPackets, yystack.l_mark[0].value, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL); 
				break;
			case DIR_OUT: 
				yyval.param.self = NewBlock(OffsetOutPackets, MaskPackets, yystack.l_mark[0].value, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL); 
				break;
			default:
				/* should never happen */
				yyerror("This token is not expected here!");
				YYABORT;
		} /* End of switch*/

	}
break;
case 12:
#line 271 "grammar.y"
	{	

		switch ( yystack.l_mark[-3].param.direction ) {
			case DIR_UNSPEC:
			case DIR_IN: 
				yyval.param.self = NewBlock(OffsetBytes, MaskBytes, yystack.l_mark[0].value, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL); 
				break;
			case DIR_OUT: 
				yyval.param.self = NewBlock(OffsetOutBytes, MaskBytes, yystack.l_mark[0].value, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL); 
				break;
			default:
				yyerror("This token is not expected here!");
				YYABORT;
		} /* End of switch*/

	}
break;
case 13:
#line 288 "grammar.y"
	{	
			yyval.param.self = NewBlock(OffsetAggrFlows, MaskFlows, yystack.l_mark[0].value, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL); 
	}
break;
case 14:
#line 292 "grammar.y"
	{	
		yyval.param.self = NewBlock(0, AnyMask, yystack.l_mark[0].value, yystack.l_mark[-1].param.comp, FUNC_PPS, NULL); 
	}
break;
case 15:
#line 296 "grammar.y"
	{	
		yyval.param.self = NewBlock(0, AnyMask, yystack.l_mark[0].value, yystack.l_mark[-1].param.comp, FUNC_BPS, NULL); 
	}
break;
case 16:
#line 300 "grammar.y"
	{	
		yyval.param.self = NewBlock(0, AnyMask, yystack.l_mark[0].value, yystack.l_mark[-1].param.comp, FUNC_BPP, NULL); 
	}
break;
case 17:
#line 304 "grammar.y"
	{	
		yyval.param.self = NewBlock(0, AnyMask, yystack.l_mark[0].value, yystack.l_mark[-1].param.comp, FUNC_DURATION, NULL); 
	}
break;
case 18:
#line 308 "grammar.y"
	{	
		yyval.param.self = NewBlock(OffsetFlowCount, MaskFlowCount, (yystack.l_mark[0].value << ShiftFlowCount) & MaskFlowCount, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL); 
	}
break;
case 19:
#line 312 "grammar.y"
	{	
		if ( yystack.l_mark[0].value > 255 ) {
			yyerror("TOS must be 0..255");
			YYABORT;
		}

		switch ( yystack.l_mark[-3].param.direction ) {
			case DIR_UNSPEC:
			case SOURCE:
				yyval.param.self = NewBlock(OffsetTos, MaskTos, (yystack.l_mark[0].value << ShiftTos) & MaskTos, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL); 
				break;
			case DESTINATION:
				yyval.param.self = NewBlock(OffsetDstTos, MaskDstTos, (yystack.l_mark[0].value << ShiftDstTos) & MaskDstTos, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL); 
				break;
			case SOURCE_OR_DESTINATION: 
				yyval.param.self = Connect_OR(
					NewBlock(OffsetTos, MaskTos, (yystack.l_mark[0].value << ShiftTos) & MaskTos, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL),
					NewBlock(OffsetDstTos, MaskDstTos, (yystack.l_mark[0].value << ShiftDstTos) & MaskDstTos, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL)
				);
				break;
			case SOURCE_AND_DESTINATION:
				yyval.param.self = Connect_AND(
					NewBlock(OffsetTos, MaskTos, (yystack.l_mark[0].value << ShiftTos) & MaskTos, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL),
					NewBlock(OffsetDstTos, MaskDstTos, (yystack.l_mark[0].value << ShiftDstTos) & MaskDstTos, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL)
				);
				break;
			default:
				yyerror("This token is not expected here!");
				YYABORT;
			}
	}
break;
case 20:
#line 344 "grammar.y"
	{	
		if ( yystack.l_mark[0].value > 10 ) {
			yyerror("Netflow version must be <= 10");
			YYABORT;
		}
		yyval.param.self = NewBlock(OffsetRecordVersion, MaskRecordVersion, (yystack.l_mark[0].value << ShiftRecordVersion) & MaskRecordVersion, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL); 
	}
break;
case 21:
#line 352 "grammar.y"
	{	
		if ( yystack.l_mark[0].value > 63 ) {
			yyerror("Flags must be 0..63");
			YYABORT;
		}
		yyval.param.self = Connect_AND(
			/* imply flags with proto TCP*/
			NewBlock(OffsetProto, MaskProto, ((uint64_t)IPPROTO_TCP << ShiftProto) & MaskProto, CMP_EQ, FUNC_NONE, NULL),
			NewBlock(OffsetFlags, MaskFlags, (yystack.l_mark[0].value << ShiftFlags) & MaskFlags, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL)
		);
	}
break;
case 22:
#line 364 "grammar.y"
	{	
		/* handle special case with 'AS' takes as flags. and not AS number*/
		uint64_t fl = 0;
		fl |= 16;
		fl |= 2;
		yyval.param.self = Connect_AND(
			/* imply flags with proto TCP */
			NewBlock(OffsetProto, MaskProto, ((uint64_t)IPPROTO_TCP << ShiftProto) & MaskProto, CMP_EQ, FUNC_NONE, NULL),
			NewBlock(OffsetFlags, (fl << ShiftFlags) & MaskFlags, (fl << ShiftFlags) & MaskFlags, CMP_FLAGS, FUNC_NONE, NULL)
		);
	}
break;
case 23:
#line 376 "grammar.y"
	{	
		uint64_t fl = 0;
		int cnt     = 0;
		size_t		len = strlen(yystack.l_mark[0].s);

		if ( len > 7 ) {
			yyerror("Too many flags");
			YYABORT;
		}

		if ( strchr(yystack.l_mark[0].s, 'F') ) { fl |=  1; cnt++; }
		if ( strchr(yystack.l_mark[0].s, 'S') ) { fl |=  2; cnt++; }
		if ( strchr(yystack.l_mark[0].s, 'R') ) { fl |=  4; cnt++; }
		if ( strchr(yystack.l_mark[0].s, 'P') ) { fl |=  8; cnt++; }
		if ( strchr(yystack.l_mark[0].s, 'A') ) { fl |=  16; cnt++; }
		if ( strchr(yystack.l_mark[0].s, 'U') ) { fl |=  32; cnt++; }
		if ( strchr(yystack.l_mark[0].s, 'X') ) { fl =  63; cnt++; }

		if ( cnt != len ) {
			yyerror("Too many flags");
			YYABORT;
		}

		yyval.param.self = Connect_AND(
			/* imply flags with proto TCP*/
			NewBlock(OffsetProto, MaskProto, ((uint64_t)IPPROTO_TCP << ShiftProto) & MaskProto, CMP_EQ, FUNC_NONE, NULL),
			NewBlock(OffsetFlags, (fl << ShiftFlags) & MaskFlags, (fl << ShiftFlags) & MaskFlags, CMP_FLAGS, FUNC_NONE, NULL)
		);
	}
break;
case 24:
#line 406 "grammar.y"
	{ 	
		int af, bytes, ret;

		ret = parse_ip(&af, yystack.l_mark[0].s, IPstack, &bytes, ALLOW_LOOKUP, &num_ip);

		if ( ret == 0 ) {
			yyerror("Error parsing IP address.");
			YYABORT;
		}

		/* ret == -1 will never happen here, as ALLOW_LOOKUP is set*/
		if ( ret == -2 ) {
			/* could not resolv host => 'not any'*/
			yyval.param.self = Invert(NewBlock(OffsetProto, 0, 0, CMP_EQ, FUNC_NONE, NULL )); 
		} else {
			uint64_t offsets[4] = {OffsetSrcIPv6a, OffsetSrcIPv6b, OffsetDstIPv6a, OffsetDstIPv6b };
			if ( af && (( af == PF_INET && bytes != 4 ) || ( af == PF_INET6 && bytes != 16 ))) {
				yyerror("incomplete IP address");
				YYABORT;
			}

			switch ( yystack.l_mark[-2].param.direction ) {
				case SOURCE:
				case DESTINATION:
					yyval.param.self = ChainHosts(offsets, IPstack, num_ip, yystack.l_mark[-2].param.direction);
					break;
				case DIR_UNSPEC:
				case SOURCE_OR_DESTINATION: {
					uint32_t src = ChainHosts(offsets, IPstack, num_ip, SOURCE);
					uint32_t dst = ChainHosts(offsets, IPstack, num_ip, DESTINATION);
					yyval.param.self = Connect_OR(src, dst);
					} break;
				case SOURCE_AND_DESTINATION: {
					uint32_t src = ChainHosts(offsets, IPstack, num_ip, SOURCE);
					uint32_t dst = ChainHosts(offsets, IPstack, num_ip, DESTINATION);
					yyval.param.self = Connect_AND(src, dst);
					} break;
				default:
					yyerror("This token is not expected here!");
					YYABORT;
	
			} /* End of switch*/

		}
	}
break;
case 25:
#line 452 "grammar.y"
	{ 	
		int af, bytes, ret;

		ret = parse_ip(&af, yystack.l_mark[0].s, IPstack, &bytes, ALLOW_LOOKUP, &num_ip);

		if ( ret == 0 ) {
			yyerror("Error parsing IP address.");
			YYABORT;
		}

		/* ret == -1 will never happen here, as ALLOW_LOOKUP is set*/
		if ( ret == -2 ) {
			/* could not resolv host => 'not any'*/
			yyval.param.self = Invert(NewBlock(OffsetProto, 0, 0, CMP_EQ, FUNC_NONE, NULL )); 
		} else {
			uint64_t offsets[4] = {OffsetTUNSRCIP, OffsetTUNSRCIP+1, OffsetTUNDSTIP, OffsetTUNDSTIP+1 };
			if ( af && (( af == PF_INET && bytes != 4 ) || ( af == PF_INET6 && bytes != 16 ))) {
				yyerror("incomplete IP address");
				YYABORT;
			}

			switch ( yystack.l_mark[-2].param.direction ) {
				case SOURCE:
				case DESTINATION:
					yyval.param.self = ChainHosts(offsets, IPstack, num_ip, yystack.l_mark[-2].param.direction);
					break;
				case DIR_UNSPEC:
				case SOURCE_OR_DESTINATION: {
					uint32_t src = ChainHosts(offsets, IPstack, num_ip, SOURCE);
					uint32_t dst = ChainHosts(offsets, IPstack, num_ip, DESTINATION);
					yyval.param.self = Connect_OR(src, dst);
					} break;
				case SOURCE_AND_DESTINATION: {
					uint32_t src = ChainHosts(offsets, IPstack, num_ip, SOURCE);
					uint32_t dst = ChainHosts(offsets, IPstack, num_ip, DESTINATION);
					yyval.param.self = Connect_AND(src, dst);
					} break;
				default:
					yyerror("This token is not expected here!");
					YYABORT;
	
			} /* End of switch*/

		}
	}
break;
case 26:
#line 498 "grammar.y"
	{ 	

		switch ( yystack.l_mark[-5].param.direction ) {
			case SOURCE:
				yyval.param.self = NewBlock(OffsetSrcIPv6a, MaskIPv6, 0 , CMP_IPLIST, FUNC_NONE, (void *)yystack.l_mark[-1].list );
				break;
			case DESTINATION:
				yyval.param.self = NewBlock(OffsetDstIPv6a, MaskIPv6, 0 , CMP_IPLIST, FUNC_NONE, (void *)yystack.l_mark[-1].list );
				break;
			case DIR_UNSPEC:
			case SOURCE_OR_DESTINATION:
				yyval.param.self = Connect_OR(
					NewBlock(OffsetSrcIPv6a, MaskIPv6, 0 , CMP_IPLIST, FUNC_NONE, (void *)yystack.l_mark[-1].list ),
					NewBlock(OffsetDstIPv6a, MaskIPv6, 0 , CMP_IPLIST, FUNC_NONE, (void *)yystack.l_mark[-1].list )
				);
				break;
			case SOURCE_AND_DESTINATION:
				yyval.param.self = Connect_AND(
					NewBlock(OffsetSrcIPv6a, MaskIPv6, 0 , CMP_IPLIST, FUNC_NONE, (void *)yystack.l_mark[-1].list ),
					NewBlock(OffsetDstIPv6a, MaskIPv6, 0 , CMP_IPLIST, FUNC_NONE, (void *)yystack.l_mark[-1].list )
				);
				break;
			default:
				yyerror("This token is not expected here!");
				YYABORT;
		}
	}
break;
case 27:
#line 526 "grammar.y"
	{ 	
		int af, bytes, ret;

		ret = parse_ip(&af, yystack.l_mark[0].s, IPstack, &bytes, STRICT_IP, &num_ip);

		if ( ret == 0 ) {
			yyerror("Error parsing IP address.");
			YYABORT;
		}

		if ( ret == -1 ) {
			yyerror("IP address required - hostname not allowed here.");
			YYABORT;
		}
		/* ret == -2 will never happen here, as STRICT_IP is set*/

		if ( af && (( af == PF_INET && bytes != 4 ) || ( af == PF_INET6 && bytes != 16 ))) {
			yyerror("incomplete IP address");
			YYABORT;
		}

		yyval.param.self = Connect_AND(
			NewBlock(OffsetNexthopv6b, MaskIPv6, IPstack[1] , CMP_EQ, FUNC_NONE, NULL ),
			NewBlock(OffsetNexthopv6a, MaskIPv6, IPstack[0] , CMP_EQ, FUNC_NONE, NULL )
		);
	}
break;
case 28:
#line 553 "grammar.y"
	{ 	

		yyval.param.self = NewBlock(OffsetNexthopv6a, MaskIPv6, 0 , CMP_IPLIST, FUNC_NONE, (void *)yystack.l_mark[-1].list );

	}
break;
case 29:
#line 559 "grammar.y"
	{ 	
		int af, bytes, ret;

		ret = parse_ip(&af, yystack.l_mark[0].s, IPstack, &bytes, STRICT_IP, &num_ip);

		if ( ret == 0 ) {
			yyerror("Error parsing IP address.");
			YYABORT;
		}

		if ( ret == -1 ) {
			yyerror("IP address required - hostname not allowed here.");
			YYABORT;
		}
		/* ret == -2 will never happen here, as STRICT_IP is set*/

		if ( af && (( af == PF_INET && bytes != 4 ) || ( af == PF_INET6 && bytes != 16 ))) {
			yyerror("incomplete IP address");
			YYABORT;
		}

		yyval.param.self = Connect_AND(
			NewBlock(OffsetBGPNexthopv6b, MaskIPv6, IPstack[1] , CMP_EQ, FUNC_NONE, NULL ),
			NewBlock(OffsetBGPNexthopv6a, MaskIPv6, IPstack[0] , CMP_EQ, FUNC_NONE, NULL )
		);
	}
break;
case 30:
#line 586 "grammar.y"
	{ 	
		int af, bytes, ret;

		ret = parse_ip(&af, yystack.l_mark[0].s, IPstack, &bytes, STRICT_IP, &num_ip);

		if ( ret == 0 ) {
			yyerror("Error parsing IP address.");
			YYABORT;
		}

		if ( ret == -1 ) {
			yyerror("IP address required - hostname not allowed here.");
			YYABORT;
		}
		/* ret == -2 will never happen here, as STRICT_IP is set*/

		if ( af && (( af == PF_INET && bytes != 4 ) || ( af == PF_INET6 && bytes != 16 ))) {
			yyerror("incomplete IP address");
			YYABORT;
		}

		yyval.param.self = Connect_AND(
			NewBlock(OffsetRouterv6b, MaskIPv6, IPstack[1] , CMP_EQ, FUNC_NONE, NULL ),
			NewBlock(OffsetRouterv6a, MaskIPv6, IPstack[0] , CMP_EQ, FUNC_NONE, NULL )
		);
	}
break;
case 31:
#line 613 "grammar.y"
	{ 	
		yyval.param.self = NewBlock(OffsetClientLatency, MaskLatency, yystack.l_mark[0].value, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL); 
	}
break;
case 32:
#line 617 "grammar.y"
	{ 	
		yyval.param.self = NewBlock(OffsetServerLatency, MaskLatency, yystack.l_mark[0].value, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL); 
	}
break;
case 33:
#line 621 "grammar.y"
	{ 	
		yyval.param.self = NewBlock(OffsetAppLatency, MaskLatency, yystack.l_mark[0].value, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL); 
	}
break;
case 34:
#line 625 "grammar.y"
	{ 	
		if ( yystack.l_mark[0].value > 255 ) {
			yyerror("Router SysID expected between be 1..255");
			YYABORT;
		}
		yyval.param.self = NewBlock(OffsetExporterSysID, MaskExporterSysID, (yystack.l_mark[0].value << ShiftExporterSysID) & MaskExporterSysID, CMP_EQ, FUNC_NONE, NULL); 
	}
break;
case 35:
#line 633 "grammar.y"
	{	
		if ( yystack.l_mark[0].value > 65535 ) {
			yyerror("Port outside of range 0..65535");
			YYABORT;
		}

		switch ( yystack.l_mark[-3].param.direction ) {
			case SOURCE:
				yyval.param.self = NewBlock(OffsetPort, MaskSrcPort, (yystack.l_mark[0].value << ShiftSrcPort) & MaskSrcPort, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL );
				break;
			case DESTINATION:
				yyval.param.self = NewBlock(OffsetPort, MaskDstPort, (yystack.l_mark[0].value << ShiftDstPort) & MaskDstPort, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL );
				break;
			case DIR_UNSPEC:
			case SOURCE_OR_DESTINATION:
				yyval.param.self = Connect_OR(
					NewBlock(OffsetPort, MaskSrcPort, (yystack.l_mark[0].value << ShiftSrcPort) & MaskSrcPort, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL ),
					NewBlock(OffsetPort, MaskDstPort, (yystack.l_mark[0].value << ShiftDstPort) & MaskDstPort, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL )
				);
				break;
			case SOURCE_AND_DESTINATION:
				yyval.param.self = Connect_AND(
					NewBlock(OffsetPort, MaskSrcPort, (yystack.l_mark[0].value << ShiftSrcPort) & MaskSrcPort, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL ),
					NewBlock(OffsetPort, MaskDstPort, (yystack.l_mark[0].value << ShiftDstPort) & MaskDstPort, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL )
				);
				break;
			default:
				yyerror("This token is not expected here!");
				YYABORT;
		} /* End switch*/

	}
break;
case 36:
#line 666 "grammar.y"
	{
		if ( yystack.l_mark[0].value > 0xFFFFFFFFLL ) {
			yyerror("Invalid ingress vrf ID");
			YYABORT;
		}
		yyval.param.self = NewBlock(OffsetIVRFID, MaskIVRFID, ( yystack.l_mark[0].value << ShiftIVRFID) & MaskIVRFID, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL );
	}
break;
case 37:
#line 674 "grammar.y"
	{
		if ( yystack.l_mark[0].value > 0xFFFFFFFFLL ) {
			yyerror("Invalid egress vrf ID");
			YYABORT;
		}
		yyval.param.self = NewBlock(OffsetEVRFID, MaskEVRFID, ( yystack.l_mark[0].value << ShiftEVRFID) & MaskEVRFID, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL );
	}
break;
case 38:
#line 682 "grammar.y"
	{	
#ifdef NSEL
		switch ( yystack.l_mark[-3].param.direction ) {
			case SOURCE:
					yyval.param.self = NewBlock(OffsetPort, MaskSrcPort, ShiftSrcPort, CMP_EQ, FUNC_PBLOCK, NULL );
				break;
			case DESTINATION:
					yyval.param.self = NewBlock(OffsetPort, MaskDstPort, ShiftDstPort, CMP_EQ, FUNC_PBLOCK, NULL );
				break;
			case DIR_UNSPEC:
			case SOURCE_OR_DESTINATION:
				yyval.param.self = Connect_OR(
					NewBlock(OffsetPort, MaskSrcPort, ShiftSrcPort, CMP_EQ, FUNC_PBLOCK, NULL ),
					NewBlock(OffsetPort, MaskDstPort, ShiftDstPort, CMP_EQ, FUNC_PBLOCK, NULL )
				);
				break;
			default:
				yyerror("This token is not expected here!");
				YYABORT;
		} /* End switch*/

#else
		yyerror("NAT filters not available");
		YYABORT;
#endif
	}
break;
case 39:
#line 709 "grammar.y"
	{ 	
		struct ULongListNode *node;
		ULongtree_t *root = NULL;

		if ( yystack.l_mark[-5].param.direction == DIR_UNSPEC || yystack.l_mark[-5].param.direction == SOURCE_OR_DESTINATION || yystack.l_mark[-5].param.direction == SOURCE_AND_DESTINATION ) {
			/* src and/or dst port*/
			/* we need a second rbtree due to different shifts for src and dst ports*/
			root = malloc(sizeof(ULongtree_t));

			struct ULongListNode *n;
			if ( root == NULL) {
				yyerror("malloc() error");
				YYABORT;
			}
			RB_INIT(root);

			RB_FOREACH(node, ULongtree, (ULongtree_t *)yystack.l_mark[-1].list) {
				if ( node->value > 65535 ) {
					yyerror("Port outside of range 0..65535");
					YYABORT;
				}
				if ((n = malloc(sizeof(struct ULongListNode))) == NULL) {
					yyerror("malloc() error");
					YYABORT;
				}
				n->value 	= (node->value << ShiftDstPort) & MaskDstPort;
				node->value = (node->value << ShiftSrcPort) & MaskSrcPort;
				RB_INSERT(ULongtree, root, n);
			}
		}

		switch ( yystack.l_mark[-5].param.direction ) {
			case SOURCE:
				RB_FOREACH(node, ULongtree, (ULongtree_t *)yystack.l_mark[-1].list) {
					node->value = (node->value << ShiftSrcPort) & MaskSrcPort;
				}
				yyval.param.self = NewBlock(OffsetPort, MaskSrcPort, 0, CMP_ULLIST, FUNC_NONE, (void *)yystack.l_mark[-1].list );
				break;
			case DESTINATION:
				RB_FOREACH(node, ULongtree, (ULongtree_t *)yystack.l_mark[-1].list) {
					node->value = (node->value << ShiftDstPort) & MaskDstPort;
				}
				yyval.param.self = NewBlock(OffsetPort, MaskDstPort, 0, CMP_ULLIST, FUNC_NONE, (void *)yystack.l_mark[-1].list );
				break;
			case DIR_UNSPEC:
			case SOURCE_OR_DESTINATION:
				yyval.param.self = Connect_OR(
					NewBlock(OffsetPort, MaskSrcPort, 0, CMP_ULLIST, FUNC_NONE, (void *)yystack.l_mark[-1].list ),
					NewBlock(OffsetPort, MaskDstPort, 0, CMP_ULLIST, FUNC_NONE, (void *)root )
				);
				break;
			case SOURCE_AND_DESTINATION:
				yyval.param.self = Connect_AND(
					NewBlock(OffsetPort, MaskSrcPort, 0, CMP_ULLIST, FUNC_NONE, (void *)yystack.l_mark[-1].list ),
					NewBlock(OffsetPort, MaskDstPort, 0, CMP_ULLIST, FUNC_NONE, (void *)root )
				);
				break;
			default:
				yyerror("This token is not expected here!");
				YYABORT;
		} /* End of switch*/

	}
break;
case 40:
#line 773 "grammar.y"
	{
		if ( yystack.l_mark[0].value > 255 ) {
			yyerror("ICMP type of range 0..255");
			YYABORT;
		}
		yyval.param.self = Connect_AND(
			/* imply ICMP-TYPE with a proto ICMP block*/
			Connect_OR (
				NewBlock(OffsetProto, MaskProto, ((uint64_t)IPPROTO_ICMP << ShiftProto)  & MaskProto, CMP_EQ, FUNC_NONE, NULL), 
				NewBlock(OffsetProto, MaskProto, ((uint64_t)IPPROTO_ICMPV6 << ShiftProto)  & MaskProto, CMP_EQ, FUNC_NONE, NULL)
			),
			NewBlock(OffsetICMP, MaskICMPtype, (yystack.l_mark[0].value << ShiftICMPtype) & MaskICMPtype, CMP_EQ, FUNC_NONE, NULL )
		);

	}
break;
case 41:
#line 789 "grammar.y"
	{
		if ( yystack.l_mark[0].value > 255 ) {
			yyerror("ICMP code of range 0..255");
			YYABORT;
		}
		yyval.param.self = Connect_AND(
			/* imply ICMP-CODE with a proto ICMP block*/
			Connect_OR (
				NewBlock(OffsetProto, MaskProto, ((uint64_t)IPPROTO_ICMP << ShiftProto)  & MaskProto, CMP_EQ, FUNC_NONE, NULL), 
				NewBlock(OffsetProto, MaskProto, ((uint64_t)IPPROTO_ICMPV6 << ShiftProto)  & MaskProto, CMP_EQ, FUNC_NONE, NULL)
			),
			NewBlock(OffsetICMP, MaskICMPcode, (yystack.l_mark[0].value << ShiftICMPcode) & MaskICMPcode, CMP_EQ, FUNC_NONE, NULL )
		);

	}
break;
case 42:
#line 805 "grammar.y"
	{
		if ( yystack.l_mark[0].value > 255 ) {
			yyerror("Engine type of range 0..255");
			YYABORT;
		}
		yyval.param.self = NewBlock(OffsetRouterID, MaskEngineType, (yystack.l_mark[0].value << ShiftEngineType) & MaskEngineType, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL);

	}
break;
case 43:
#line 814 "grammar.y"
	{
		if ( yystack.l_mark[0].value > 255 ) {
			yyerror("Engine ID of range 0..255");
			YYABORT;
		}
		yyval.param.self = NewBlock(OffsetRouterID, MaskEngineID, (yystack.l_mark[0].value << ShiftEngineID) & MaskEngineID, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL);

	}
break;
case 44:
#line 824 "grammar.y"
	{	
		if ( strlen(yystack.l_mark[0].s) > 16 ) {
			yyerror("Size flowlabel of range 1..16");
			YYABORT;
		}

		yyval.param.self = NewBlock(0, 0, 0, CMP_FLOWLABEL, FUNC_NONE, (void *)strdup(yystack.l_mark[0].s)); 
	}
break;
case 45:
#line 833 "grammar.y"
	{
#ifdef NSEL
		if ( strncasecmp(yystack.l_mark[0].s,"ignore", 6) == 0) {
			yyval.param.self = NewBlock(OffsetConnID, MaskFWevent, ( NSEL_EVENT_IGNORE << ShiftFWevent) & MaskFWevent, CMP_EQ, FUNC_NONE, NULL );
		} else if( strncasecmp(yystack.l_mark[0].s,"create", 6) == 0) {
			yyval.param.self = NewBlock(OffsetConnID, MaskFWevent, ( NSEL_EVENT_CREATE << ShiftFWevent) & MaskFWevent, CMP_EQ, FUNC_NONE, NULL );
		} else if( strncasecmp(yystack.l_mark[0].s,"term", 4) == 0 || strncasecmp(yystack.l_mark[0].s,"delete", 6) == 0) {
			yyval.param.self = NewBlock(OffsetConnID, MaskFWevent, ( NSEL_EVENT_DELETE << ShiftFWevent) & MaskFWevent, CMP_EQ, FUNC_NONE, NULL );
		} else if  (strncasecmp(yystack.l_mark[0].s,"deny", 4) == 0) {
			yyval.param.self = NewBlock(OffsetConnID, MaskFWevent, ( NSEL_EVENT_DENIED << ShiftFWevent) & MaskFWevent, CMP_EQ, FUNC_NONE, NULL );
		} else if  (strncasecmp(yystack.l_mark[0].s,"alert", 5) == 0) {
			yyval.param.self = NewBlock(OffsetConnID, MaskFWevent, ( NSEL_EVENT_ALERT << ShiftFWevent) & MaskFWevent, CMP_EQ, FUNC_NONE, NULL );
		} else if  (strncasecmp(yystack.l_mark[0].s,"update", 6) == 0) {
			yyval.param.self = NewBlock(OffsetConnID, MaskFWevent, ( NSEL_EVENT_UPDATE << ShiftFWevent) & MaskFWevent, CMP_EQ, FUNC_NONE, NULL );
		} else {
			yyerror("Unknown asa event");
			YYABORT;
		}
#else
		yyerror("NSEL/ASA filters not available");
		YYABORT;
#endif
	}
break;
case 46:
#line 857 "grammar.y"
	{
#ifdef NSEL
		if ( yystack.l_mark[0].value > 255 ) {
			yyerror("Invalid xevent ID");
			YYABORT;
		}
		yyval.param.self = NewBlock(OffsetConnID, MaskFWevent, ( yystack.l_mark[0].value << ShiftFWevent) & MaskFWevent, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL );
#else
		yyerror("NSEL/ASA filters not available");
		YYABORT;
#endif
	}
break;
case 47:
#line 870 "grammar.y"
	{
#ifdef NSEL
		uint64_t xevent = 0;
		if ( yystack.l_mark[0].param.inout == INGRESS ) {
			xevent = 1001;
		} else if ( yystack.l_mark[0].param.inout == EGRESS ) {
			xevent = 1002;
		} else {
				yyerror("Invalid inout token");
				YYABORT;
		}
		yyval.param.self = Connect_AND(
			NewBlock(OffsetConnID, MaskFWevent, ( NSEL_EVENT_DENIED << ShiftFWevent) & MaskFWevent, CMP_EQ, FUNC_NONE, NULL ),
			NewBlock(OffsetConnID, MaskFWXevent, ( xevent << ShiftFWXevent) & MaskFWXevent, CMP_EQ, FUNC_NONE, NULL )
		);
#else
		yyerror("NSEL/ASA filters not available");
		YYABORT;
#endif
	}
break;
case 48:
#line 890 "grammar.y"
	{
#ifdef NSEL
		uint64_t xevent = 0;
		if( strncasecmp(yystack.l_mark[0].s,"interface", 9) == 0) {
			xevent = 1003;
		} else if( strncasecmp(yystack.l_mark[0].s,"nosyn", 5) == 0) {
			xevent = 1004;
		} else {
			xevent = (uint64_t)strtol(yystack.l_mark[0].s, (char **)NULL, 10);
			if ( (xevent == 0 && errno == EINVAL) || xevent > 65535 ) {
				yyerror("Invalid xevent ID");
				YYABORT;
			}
		}
		yyval.param.self = Connect_AND(
			NewBlock(OffsetConnID, MaskFWevent, ( NSEL_EVENT_DENIED << ShiftFWevent) & MaskFWevent, CMP_EQ, FUNC_NONE, NULL ),
			NewBlock(OffsetConnID, MaskFWXevent, ( xevent << ShiftFWXevent) & MaskFWXevent, CMP_EQ, FUNC_NONE, NULL )
		);
#else
		yyerror("NSEL/ASA filters not available");
		YYABORT;
#endif
	}
break;
case 49:
#line 914 "grammar.y"
	{
#ifdef NSEL
		if ( yystack.l_mark[0].value > 65535 ) {
			yyerror("Invalid xevent ID");
			YYABORT;
		}
		yyval.param.self = NewBlock(OffsetConnID, MaskFWXevent, ( yystack.l_mark[0].value << ShiftFWXevent) & MaskFWXevent, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL );
#else
		yyerror("NSEL/ASA filters not available");
		YYABORT;
#endif
	}
break;
case 50:
#line 927 "grammar.y"
	{ 
#ifdef NSEL
		int af, bytes, ret;
		uint64_t	mask[2];

		ret = parse_ip(&af, yystack.l_mark[-2].s, IPstack, &bytes, STRICT_IP, &num_ip);
		if ( ret == 0 ) {
			yyerror("Invalid IP address");
			YYABORT;
		}
		if ( ret == -1 ) {
			yyerror("IP address required - hostname not allowed here.");
			YYABORT;
		}
		/* ret == -2 will never happen here, as STRICT_IP is set*/


		if ( yystack.l_mark[0].value > (bytes*8) ) {
			yyerror("Too many netbits for this IP address");
			YYABORT;
		}

		if ( af == PF_INET ) {
			mask[0] = 0xffffffffffffffffLL;
			mask[1] = 0xffffffffffffffffLL << ( 32 - yystack.l_mark[0].value );
		} else {	/* PF_INET6*/
			if ( yystack.l_mark[0].value > 64 ) {
				mask[0] = 0xffffffffffffffffLL;
				mask[1] = 0xffffffffffffffffLL << ( 128 - yystack.l_mark[0].value );
			} else {
				mask[0] = 0xffffffffffffffffLL << ( 64 - yystack.l_mark[0].value );
				mask[1] = 0;
			}
		}
		/* IP aadresses are stored in network representation */
		mask[0]	 = mask[0];
		mask[1]	 = mask[1];

		IPstack[0] &= mask[0];
		IPstack[1] &= mask[1];

		switch ( yystack.l_mark[-4].param.direction ) {
			case SOURCE:
				yyval.param.self = Connect_AND(
					NewBlock(OffsetXLATESRCv6b, mask[1], IPstack[1] , CMP_EQ, FUNC_NONE, NULL ),
					NewBlock(OffsetXLATESRCv6a, mask[0], IPstack[0] , CMP_EQ, FUNC_NONE, NULL )
				);
				break;
			case DESTINATION:
				yyval.param.self = Connect_AND(
					NewBlock(OffsetXLATEDSTv6b, mask[1], IPstack[1] , CMP_EQ, FUNC_NONE, NULL ),
					NewBlock(OffsetXLATEDSTv6a, mask[0], IPstack[0] , CMP_EQ, FUNC_NONE, NULL )
				);
				break;
			case DIR_UNSPEC:
			case SOURCE_OR_DESTINATION:
				yyval.param.self = Connect_OR(
					Connect_AND(
						NewBlock(OffsetXLATESRCv6b, mask[1], IPstack[1] , CMP_EQ, FUNC_NONE, NULL ),
						NewBlock(OffsetXLATESRCv6a, mask[0], IPstack[0] , CMP_EQ, FUNC_NONE, NULL )
					),
					Connect_AND(
						NewBlock(OffsetXLATEDSTv6b, mask[1], IPstack[1] , CMP_EQ, FUNC_NONE, NULL ),
						NewBlock(OffsetXLATEDSTv6a, mask[0], IPstack[0] , CMP_EQ, FUNC_NONE, NULL )
					)
				);
				break;
			case SOURCE_AND_DESTINATION:
				yyval.param.self = Connect_AND(
					Connect_AND(
						NewBlock(OffsetXLATESRCv6b, mask[1], IPstack[1] , CMP_EQ, FUNC_NONE, NULL ),
						NewBlock(OffsetXLATESRCv6a, mask[0], IPstack[0] , CMP_EQ, FUNC_NONE, NULL )
					),
					Connect_AND(
						NewBlock(OffsetXLATEDSTv6b, mask[1], IPstack[1] , CMP_EQ, FUNC_NONE, NULL ),
						NewBlock(OffsetXLATEDSTv6a, mask[0], IPstack[0] , CMP_EQ, FUNC_NONE, NULL )
					)
				);
				break;
			default:
				yyerror("This token is not expected here!");
				YYABORT;
		} /* End of switch*/

#else
		yyerror("NSEL/ASA filters not available");
		YYABORT;
#endif
	}
break;
case 51:
#line 1017 "grammar.y"
	{	
#ifdef NSEL
		if ( yystack.l_mark[0].value > 65535 ) {
			yyerror("Port outside of range 0..65535");
			YYABORT;
		}

		switch ( yystack.l_mark[-3].param.direction ) {
			case SOURCE:
				yyval.param.self = NewBlock(OffsetXLATEPort, MaskXLATESRCPORT, (yystack.l_mark[0].value << ShiftXLATESRCPORT) & MaskXLATESRCPORT, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL );
				break;
			case DESTINATION:
				yyval.param.self = NewBlock(OffsetXLATEPort, MaskXLATEDSTPORT, (yystack.l_mark[0].value << ShiftXLATEDSTPORT) & MaskXLATEDSTPORT, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL );
				break;
			case DIR_UNSPEC:
			case SOURCE_OR_DESTINATION:
				yyval.param.self = Connect_OR(
					NewBlock(OffsetXLATEPort, MaskXLATESRCPORT, (yystack.l_mark[0].value << ShiftXLATESRCPORT) & MaskXLATESRCPORT, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL ),
					NewBlock(OffsetXLATEPort, MaskXLATEDSTPORT, (yystack.l_mark[0].value << ShiftXLATEDSTPORT) & MaskXLATEDSTPORT, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL )
				);
				break;
			case SOURCE_AND_DESTINATION:
				yyval.param.self = Connect_AND(
					NewBlock(OffsetXLATEPort, MaskXLATESRCPORT, (yystack.l_mark[0].value << ShiftXLATESRCPORT) & MaskXLATESRCPORT, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL ),
					NewBlock(OffsetXLATEPort, MaskXLATEDSTPORT, (yystack.l_mark[0].value << ShiftXLATEDSTPORT) & MaskXLATEDSTPORT, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL )
				);
				break;
			default:
				yyerror("This token is not expected here!");
				YYABORT;
		} /* End switch*/
#else
		yyerror("NSEL/ASA filters not available");
		YYABORT;
#endif

	}
break;
case 52:
#line 1055 "grammar.y"
	{
#ifdef NSEL
		uint64_t offset, mask, shift;
		if ( yystack.l_mark[-3].param.inout == INGRESS ) {
			switch (yystack.l_mark[-2].param.acl) {
				case ACL:
					offset = OffsetIngressAclId;
					mask   = MaskIngressAclId;	
					shift  = ShiftIngressAclId;
					break;
				case ACE:
					offset = OffsetIngressAceId;
					mask   = MaskIngressAceId;	
					shift  = ShiftIngressAceId;
					break;
				case XACE:
					offset = OffsetIngressGrpId;
					mask   = MaskIngressGrpId;	
					shift  = ShiftIngressGrpId;
					break;
				default:
					yyerror("Invalid ACL specifier");
					YYABORT;
			}
		} else if ( yystack.l_mark[-3].param.inout == EGRESS && yyval.param.acl == ACL ) {
			offset = OffsetEgressAclId;
			mask   = MaskEgressAclId;	
			shift  = ShiftEgressAclId;
		} else {
			yyerror("ingress/egress syntax error");
			YYABORT;
		}
		yyval.param.self = NewBlock(offset, mask, (yystack.l_mark[0].value << shift) & mask , yystack.l_mark[-1].param.comp, FUNC_NONE, NULL );

#else
		yyerror("NSEL/ASA filters not available");
		YYABORT;
#endif
	}
break;
case 53:
#line 1095 "grammar.y"
	{
#ifdef NSEL
		if ( strncasecmp(yystack.l_mark[0].s,"invalid", 7) == 0) {
			yyval.param.self = NewBlock(OffsetNATevent, MasNATevent, ( NEL_EVENT_INVALID << ShiftNATevent) & MasNATevent, CMP_EQ, FUNC_NONE, NULL );
		} else if( strncasecmp(yystack.l_mark[0].s,"add", 3) == 0 || strncasecmp(yystack.l_mark[0].s,"create", 6) == 0) {
			yyval.param.self = NewBlock(OffsetNATevent, MasNATevent, ( NEL_EVENT_ADD << ShiftNATevent) & MasNATevent, CMP_EQ, FUNC_NONE, NULL );
		} else if( strncasecmp(yystack.l_mark[0].s,"delete", 6) == 0) {
			yyval.param.self = NewBlock(OffsetNATevent, MasNATevent, ( NEL_EVENT_DELETE << ShiftNATevent) & MasNATevent, CMP_EQ, FUNC_NONE, NULL );
		} else {
			yyerror("Unknown nat event");
			YYABORT;
		}
#else
		yyerror("NAT filters not available");
		YYABORT;
#endif
	}
break;
case 54:
#line 1113 "grammar.y"
	{
#ifdef NSEL
		if ( yystack.l_mark[0].value > 255 ) {
			yyerror("Invalid event ID");
			YYABORT;
		}
		yyval.param.self = NewBlock(OffsetNATevent, MasNATevent, ( yystack.l_mark[0].value << ShiftNATevent) & MasNATevent, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL );
#else
		yyerror("NAT filters not available");
		YYABORT;
#endif
	}
break;
case 55:
#line 1128 "grammar.y"
	{
#ifdef NSEL
		if ( yystack.l_mark[0].value > 65536 ) {
			yyerror("Invalid port");
			YYABORT;
		}
		yyval.param.self = NewBlock(OffsetPortBlock, MaskPortBlockStart, ( yystack.l_mark[0].value << ShiftPortBlockStart) & MaskPortBlockStart, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL );
#else
		yyerror("NAT filters not available");
		YYABORT;
#endif
	}
break;
case 56:
#line 1141 "grammar.y"
	{
#ifdef NSEL
		if ( yystack.l_mark[0].value > 65536 ) {
			yyerror("Invalid port");
			YYABORT;
		}
		yyval.param.self = NewBlock(OffsetPortBlock, MaskPortBlockEnd, ( yystack.l_mark[0].value << ShiftPortBlockEnd) & MaskPortBlockEnd, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL );
#else
		yyerror("NAT filters not available");
		YYABORT;
#endif
	}
break;
case 57:
#line 1154 "grammar.y"
	{
#ifdef NSEL
		if ( yystack.l_mark[0].value > 65536 ) {
			yyerror("Invalid port");
			YYABORT;
		}
		yyval.param.self = NewBlock(OffsetPortBlock, MaskPortBlockStep, ( yystack.l_mark[0].value << ShiftPortBlockStep) & MaskPortBlockStep, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL );
#else
		yyerror("NAT filters not available");
		YYABORT;
#endif
	}
break;
case 58:
#line 1167 "grammar.y"
	{
#ifdef NSEL
		if ( yystack.l_mark[0].value > 65536 ) {
			yyerror("Invalid port");
			YYABORT;
		}
		yyval.param.self = NewBlock(OffsetPortBlock, MaskPortBlockSize, ( yystack.l_mark[0].value << ShiftPortBlockSize) & MaskPortBlockSize, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL );
#else
		yyerror("NAT filters not available");
		YYABORT;
#endif
	}
break;
case 59:
#line 1180 "grammar.y"
	{	
#ifdef NSEL
		if ( yystack.l_mark[0].value > 65535 ) {
			yyerror("Port outside of range 0..65535");
			YYABORT;
		}

		switch ( yystack.l_mark[-3].param.direction ) {
			case SOURCE:
				yyval.param.self = NewBlock(OffsetXLATEPort, MaskXLATESRCPORT, (yystack.l_mark[0].value << ShiftXLATESRCPORT) & MaskXLATESRCPORT, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL );
				break;
			case DESTINATION:
				yyval.param.self = NewBlock(OffsetXLATEPort, MaskXLATEDSTPORT, (yystack.l_mark[0].value << ShiftXLATEDSTPORT) & MaskXLATEDSTPORT, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL );
				break;
			case DIR_UNSPEC:
			case SOURCE_OR_DESTINATION:
				yyval.param.self = Connect_OR(
					NewBlock(OffsetXLATEPort, MaskXLATESRCPORT, (yystack.l_mark[0].value << ShiftXLATESRCPORT) & MaskXLATESRCPORT, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL ),
					NewBlock(OffsetXLATEPort, MaskXLATEDSTPORT, (yystack.l_mark[0].value << ShiftXLATEDSTPORT) & MaskXLATEDSTPORT, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL )
				);
				break;
			case SOURCE_AND_DESTINATION:
				yyval.param.self = Connect_AND(
					NewBlock(OffsetXLATEPort, MaskXLATESRCPORT, (yystack.l_mark[0].value << ShiftXLATESRCPORT) & MaskXLATESRCPORT, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL ),
					NewBlock(OffsetXLATEPort, MaskXLATEDSTPORT, (yystack.l_mark[0].value << ShiftXLATEDSTPORT) & MaskXLATEDSTPORT, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL )
				);
				break;
			default:
				yyerror("This token is not expected here!");
				YYABORT;
		} /* End switch*/
#else
		yyerror("NEL/NAT filters not available");
		YYABORT;
#endif

	}
break;
case 60:
#line 1218 "grammar.y"
	{ 	
#ifdef NSEL
		int af, bytes, ret;

		ret = parse_ip(&af, yystack.l_mark[0].s, IPstack, &bytes, ALLOW_LOOKUP, &num_ip);

		if ( ret == 0 ) {
			yyerror("Error parsing IP address.");
			YYABORT;
		}

		/* ret == -1 will never happen here, as ALLOW_LOOKUP is set*/
		if ( ret == -2 ) {
			/* could not resolv host => 'not any'*/
			yyval.param.self = Invert(NewBlock(OffsetProto, 0, 0, CMP_EQ, FUNC_NONE, NULL )); 
		} else {
			uint64_t offsets[4] = {OffsetXLATESRCv6a, OffsetXLATESRCv6b, OffsetXLATEDSTv6a, OffsetXLATEDSTv6b };
			if ( af && (( af == PF_INET && bytes != 4 ) || ( af == PF_INET6 && bytes != 16 ))) {
				yyerror("incomplete IP address");
				YYABORT;
			}

			switch ( yystack.l_mark[-2].param.direction ) {
				case SOURCE:
				case DESTINATION:
					yyval.param.self = ChainHosts(offsets, IPstack, num_ip, yystack.l_mark[-2].param.direction);
					break;
				case DIR_UNSPEC:
				case SOURCE_OR_DESTINATION: {
					uint32_t src = ChainHosts(offsets, IPstack, num_ip, SOURCE);
					uint32_t dst = ChainHosts(offsets, IPstack, num_ip, DESTINATION);
					yyval.param.self = Connect_OR(src, dst);
					} break;
				case SOURCE_AND_DESTINATION: {
					uint32_t src = ChainHosts(offsets, IPstack, num_ip, SOURCE);
					uint32_t dst = ChainHosts(offsets, IPstack, num_ip, DESTINATION);
					yyval.param.self = Connect_AND(src, dst);
					} break;
				default:
					yyerror("This token is not expected here!");
					YYABORT;
	
			} /* End of switch*/

		}
#else
		yyerror("NSEL/ASA filters not available");
		YYABORT;
#endif
	}
break;
case 61:
#line 1270 "grammar.y"
	{
		if (strlen(yystack.l_mark[0].s)>64) {
			yyerror("word too long");
			YYABORT;
		}
		yyval.param.self = NewBlock(OffsetPayload, 0, 0, CMP_PAYLOAD, FUNC_NONE, yystack.l_mark[0].s); 
	}
break;
case 62:
#line 1278 "grammar.y"
	{
		if (strlen(yystack.l_mark[0].s)>64) {
			yyerror("word too long");
			YYABORT;
		}
		char *word = stripWord(yystack.l_mark[0].s);
		yyval.param.self = NewBlock(OffsetPayload, 0, 0, CMP_PAYLOAD, FUNC_NONE, word); 
	}
break;
case 63:
#line 1287 "grammar.y"
	{
		if (strlen(yystack.l_mark[0].s)>64) {
			yyerror("word too long");
			YYABORT;
		}
		/* strip ' or " */
		char *word = stripWord(yystack.l_mark[0].s);

		int err[2];
		srx_Context *program = srx_CreateExt(word, strlen(word), "", err, NULL, NULL);
		if ( !program ) {
			yyerror("failed to compile regex");
		}

		yyval.param.self = NewBlock(OffsetPayload, 0, 0, CMP_REGEX, FUNC_NONE, (char *)program); 
	}
break;
case 64:
#line 1304 "grammar.y"
	{
		if (strlen(yystack.l_mark[-1].s)>64) {
			yyerror("word too long");
			YYABORT;
		}
		/* strip ' or " */
		char *word = stripWord(yystack.l_mark[-1].s);

		int err[2];
		srx_Context *program = srx_CreateExt(word, strlen(word), yystack.l_mark[0].s, err, NULL, NULL);
		if ( !program ) {
			yyerror("failed to compile regex");
		}

		yyval.param.self = NewBlock(OffsetPayload, 0, 0, CMP_REGEX, FUNC_NONE, (char *)program); 
	}
break;
case 65:
#line 1321 "grammar.y"
	{
		union {
			uint8_t u8[16];
			uint64_t u64[2];
		} ja3;

		if ( strcasecmp(yystack.l_mark[0].s, "defined") == 0) {
			ja3.u64[0] = 0;
			ja3.u64[1] = 0;
			yyval.param.self = Invert(Connect_AND(
				NewBlock(OffsetJA3, MaskJA3, ja3.u64[0], CMP_EQ, FUNC_NONE, NULL ),
				NewBlock(OffsetJA3+1, MaskJA3, ja3.u64[1], CMP_EQ, FUNC_NONE, NULL )
			));
		} else {
			if (!IsMD5(yystack.l_mark[0].s)) {
				yyerror("not a ja3 hash");
				YYABORT;
			}

			char *pos = yystack.l_mark[0].s;
			for(int count = 0; count < 16; count++) {
				sscanf(pos, "%2hhx", &ja3.u8[count]);
				pos += 2;
			}
			yyval.param.self = Connect_AND(
				NewBlock(OffsetJA3, MaskJA3, ja3.u64[0], CMP_EQ, FUNC_NONE, NULL ),
				NewBlock(OffsetJA3+1, MaskJA3, ja3.u64[1], CMP_EQ, FUNC_NONE, NULL )
			);
		}
		ja3Filter = 1;
	}
break;
case 66:
#line 1353 "grammar.y"
	{ 	
#ifdef NSEL
		switch ( yystack.l_mark[-5].param.direction ) {
			case SOURCE:
				yyval.param.self = NewBlock(OffsetXLATESRCv6a, MaskIPv6, 0 , CMP_IPLIST, FUNC_NONE, (void *)yystack.l_mark[-1].list );
				break;
			case DESTINATION:
				yyval.param.self = NewBlock(OffsetXLATEDSTv6a, MaskIPv6, 0 , CMP_IPLIST, FUNC_NONE, (void *)yystack.l_mark[-1].list );
				break;
			case DIR_UNSPEC:
			case SOURCE_OR_DESTINATION:
				yyval.param.self = Connect_OR(
					NewBlock(OffsetXLATESRCv6a, MaskIPv6, 0 , CMP_IPLIST, FUNC_NONE, (void *)yystack.l_mark[-1].list ),
					NewBlock(OffsetXLATEDSTv6a, MaskIPv6, 0 , CMP_IPLIST, FUNC_NONE, (void *)yystack.l_mark[-1].list )
				);
				break;
			case SOURCE_AND_DESTINATION:
				yyval.param.self = Connect_AND(
					NewBlock(OffsetXLATESRCv6a, MaskIPv6, 0 , CMP_IPLIST, FUNC_NONE, (void *)yystack.l_mark[-1].list ),
					NewBlock(OffsetXLATEDSTv6a, MaskIPv6, 0 , CMP_IPLIST, FUNC_NONE, (void *)yystack.l_mark[-1].list )
				);
				break;
			default:
				yyerror("This token is not expected here!");
				YYABORT;
		}
#else
		yyerror("NSEL/ASA filters not available");
		YYABORT;
#endif
	}
break;
case 67:
#line 1385 "grammar.y"
	{	
		if ( yystack.l_mark[0].value > 0xfFFFFFFF ) {
			yyerror("AS number of range");
			YYABORT;
		}

		geoFilter = 1;
		switch ( yystack.l_mark[-3].param.direction ) {
			case SOURCE:
				yyval.param.self = NewBlock(OffsetAS, MaskSrcAS, (yystack.l_mark[0].value << ShiftSrcAS) & MaskSrcAS, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL );
				break;
			case DESTINATION:
				yyval.param.self = NewBlock(OffsetAS, MaskDstAS, (yystack.l_mark[0].value << ShiftDstAS) & MaskDstAS, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL);
				break;
			case DIR_UNSPEC:
			case SOURCE_OR_DESTINATION:
				yyval.param.self = Connect_OR(
					NewBlock(OffsetAS, MaskSrcAS, (yystack.l_mark[0].value << ShiftSrcAS) & MaskSrcAS, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL ),
					NewBlock(OffsetAS, MaskDstAS, (yystack.l_mark[0].value << ShiftDstAS) & MaskDstAS, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL)
				);
				break;
			case SOURCE_AND_DESTINATION:
				yyval.param.self = Connect_AND(
					NewBlock(OffsetAS, MaskSrcAS, (yystack.l_mark[0].value << ShiftSrcAS) & MaskSrcAS, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL ),
					NewBlock(OffsetAS, MaskDstAS, (yystack.l_mark[0].value << ShiftDstAS) & MaskDstAS, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL)
				);
				break;
			case ADJ_PREV:
				yyval.param.self = NewBlock(OffsetBGPadj, MaskBGPadjPrev, (yystack.l_mark[0].value << ShiftBGPadjPrev) & MaskBGPadjPrev, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL );
				break;
			case ADJ_NEXT:
				yyval.param.self = NewBlock(OffsetBGPadj, MaskBGPadjNext, (yystack.l_mark[0].value << ShiftBGPadjNext) & MaskBGPadjNext, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL );
				break;
			default:
				yyerror("This token is not expected here!");
				YYABORT;
		} /* End of switch*/

	}
break;
case 68:
#line 1425 "grammar.y"
	{	
		if ( strlen(yystack.l_mark[0].s) != 2 ) {
			yyerror("Need a two letter geo country code");
			YYABORT;
		}

		geoFilter = 1;
		union {
			char c[8];
			uint64_t u;
		} v;
		v.u = 0;
		v.c[0] = toupper(yystack.l_mark[0].s[0]);
		v.c[1] = toupper(yystack.l_mark[0].s[1]);
		switch ( yystack.l_mark[-2].param.direction ) {
			case SOURCE:
				yyval.param.self = NewBlock(OffsetGeo, MaskSrcGeo, (v.u << ShiftSrcGeo) & MaskSrcGeo, CMP_EQ, FUNC_NONE, NULL );
				break;
			case DESTINATION:
				yyval.param.self = NewBlock(OffsetGeo, MaskDstGeo, (v.u << ShiftDstGeo) & MaskDstGeo, CMP_EQ, FUNC_NONE, NULL);
				break;
			case DIR_UNSPEC:
			case SOURCE_OR_DESTINATION:
				yyval.param.self = Connect_OR(
					NewBlock(OffsetGeo, MaskSrcGeo, (v.u << ShiftSrcGeo) & MaskSrcGeo, CMP_EQ, FUNC_NONE, NULL ),
					NewBlock(OffsetGeo, MaskDstGeo, (v.u << ShiftDstGeo) & MaskDstGeo, CMP_EQ, FUNC_NONE, NULL)
				);
				break;
			case SOURCE_AND_DESTINATION:
				yyval.param.self = Connect_AND(
					NewBlock(OffsetGeo, MaskSrcGeo, (v.u << ShiftSrcGeo) & MaskSrcGeo, CMP_EQ, FUNC_NONE, NULL ),
					NewBlock(OffsetGeo, MaskDstGeo, (v.u << ShiftDstGeo) & MaskDstGeo, CMP_EQ, FUNC_NONE, NULL)
				);
				break;
			default:
				yyerror("This token is not expected here!");
				YYABORT;
		} /* End of switch*/

	}
break;
case 69:
#line 1466 "grammar.y"
	{ 	
		struct ULongListNode *node;
		ULongtree_t *root = NULL;

		geoFilter = 1;
		if ( yystack.l_mark[-5].param.direction == DIR_UNSPEC || yystack.l_mark[-5].param.direction == SOURCE_OR_DESTINATION || yystack.l_mark[-5].param.direction == SOURCE_AND_DESTINATION ) {
			/* src and/or dst AS*/
			/* we need a second rbtree due to different shifts for src and dst AS*/
			root = malloc(sizeof(ULongtree_t));

			struct ULongListNode *n;
			if ( root == NULL) {
				yyerror("malloc() error");
				YYABORT;
			}
			RB_INIT(root);

			RB_FOREACH(node, ULongtree, (ULongtree_t *)yystack.l_mark[-1].list) {
				if ( node->value > 0xFFFFFFFFLL ) {
					yyerror("AS number of range");
					YYABORT;
				}
				if ((n = malloc(sizeof(struct ULongListNode))) == NULL) {
					yyerror("malloc() error");
					YYABORT;
				}
				n->value 	= (node->value << ShiftDstAS) & MaskDstAS;
				node->value = (node->value << ShiftSrcAS) & MaskSrcAS;
				RB_INSERT(ULongtree, root, n);
			}
		}

		switch ( yystack.l_mark[-5].param.direction ) {
			case SOURCE:
				RB_FOREACH(node, ULongtree, (ULongtree_t *)yystack.l_mark[-1].list) {
					node->value = (node->value << ShiftSrcAS) & MaskSrcAS;
				}
				yyval.param.self = NewBlock(OffsetAS, MaskSrcAS, 0, CMP_ULLIST, FUNC_NONE, (void *)yystack.l_mark[-1].list );
				break;
			case DESTINATION:
				RB_FOREACH(node, ULongtree, (ULongtree_t *)yystack.l_mark[-1].list) {
					node->value = (node->value << ShiftDstAS) & MaskDstAS;
				}
				yyval.param.self = NewBlock(OffsetAS, MaskDstAS, 0, CMP_ULLIST, FUNC_NONE, (void *)yystack.l_mark[-1].list );
				break;
			case DIR_UNSPEC:
			case SOURCE_OR_DESTINATION:
				yyval.param.self = Connect_OR(
					NewBlock(OffsetAS, MaskSrcAS, 0, CMP_ULLIST, FUNC_NONE, (void *)yystack.l_mark[-1].list ),
					NewBlock(OffsetAS, MaskDstAS, 0, CMP_ULLIST, FUNC_NONE, (void *)root )
				);
				break;
			case SOURCE_AND_DESTINATION:
				yyval.param.self = Connect_AND(
					NewBlock(OffsetAS, MaskSrcAS, 0, CMP_ULLIST, FUNC_NONE, (void *)yystack.l_mark[-1].list ),
					NewBlock(OffsetAS, MaskDstAS, 0, CMP_ULLIST, FUNC_NONE, (void *)root )
				);
				break;
			default:
				yyerror("This token is not expected here!");
				YYABORT;
		}

	}
break;
case 70:
#line 1531 "grammar.y"
	{	
		if ( yystack.l_mark[0].value > 255 ) {
			yyerror("Mask outside of range 0..255");
			YYABORT;
		}

		switch ( yystack.l_mark[-2].param.direction ) {
			case SOURCE:
				yyval.param.self = NewBlock(OffsetMask, MaskSrcMask, (yystack.l_mark[0].value << ShiftSrcMask) & MaskSrcMask, CMP_EQ, FUNC_NONE, NULL );
				break;
			case DESTINATION:
				yyval.param.self = NewBlock(OffsetMask, MaskDstMask, (yystack.l_mark[0].value << ShiftDstMask) & MaskDstMask, CMP_EQ, FUNC_NONE, NULL );
				break;
			case DIR_UNSPEC:
			case SOURCE_OR_DESTINATION:
				yyval.param.self = Connect_OR(
					NewBlock(OffsetMask, MaskSrcMask, (yystack.l_mark[0].value << ShiftSrcMask) & MaskSrcMask, CMP_EQ, FUNC_NONE, NULL ),
					NewBlock(OffsetMask, MaskDstMask, (yystack.l_mark[0].value << ShiftDstMask) & MaskDstMask, CMP_EQ, FUNC_NONE, NULL )
				);
				break;
			case SOURCE_AND_DESTINATION:
				yyval.param.self = Connect_AND(
					NewBlock(OffsetMask, MaskSrcMask, (yystack.l_mark[0].value << ShiftSrcMask) & MaskSrcMask, CMP_EQ, FUNC_NONE, NULL ),
					NewBlock(OffsetMask, MaskDstMask, (yystack.l_mark[0].value << ShiftDstMask) & MaskDstMask, CMP_EQ, FUNC_NONE, NULL )
				);
				break;
			default:
				yyerror("This token is not expected here!");
				YYABORT;
		} /* End switch*/

	}
break;
case 71:
#line 1564 "grammar.y"
	{ 
		int af, bytes, ret;
		uint64_t	mask[2];
		ret = parse_ip(&af, yystack.l_mark[-1].s, IPstack, &bytes, STRICT_IP, &num_ip);

		if ( ret == 0 ) {
			yyerror("Invalid IP address");
			YYABORT;
		}
		
		if ( ret == -1 ) {
			yyerror("IP address required - hostname not allowed here.");
			YYABORT;
		}
		/* ret == -2 will never happen here, as STRICT_IP is set*/

		if ( af != PF_INET ) {
			yyerror("IP netmask syntax valid only for IPv4");
			YYABORT;
		}
		if ( bytes != 4 ) {
			yyerror("Need complete IP address");
			YYABORT;
		}

		ret = parse_ip(&af, yystack.l_mark[0].s, mask, &bytes, STRICT_IP, &num_ip);
		if ( ret == 0 ) {
			yyerror("Invalid IP address");
			YYABORT;
		}
		if ( ret == -1 ) {
			yyerror("IP address required - hostname not allowed here.");
			YYABORT;
		}
		/* ret == -2 will never happen here, as STRICT_IP is set*/

		if ( af != PF_INET || bytes != 4 ) {
			yyerror("Invalid netmask for IPv4 address");
			YYABORT;
		}

		IPstack[0] &= mask[0];
		IPstack[1] &= mask[1];

		switch ( yystack.l_mark[-3].param.direction ) {
			case SOURCE:
				yyval.param.self = Connect_AND(
					NewBlock(OffsetSrcIPv6b, mask[1], IPstack[1] , CMP_EQ, FUNC_NONE, NULL ),
					NewBlock(OffsetSrcIPv6a, mask[0], IPstack[0] , CMP_EQ, FUNC_NONE, NULL )
				);
				break;
			case DESTINATION:
				yyval.param.self = Connect_AND(
					NewBlock(OffsetDstIPv6b, mask[1], IPstack[1] , CMP_EQ, FUNC_NONE, NULL ),
					NewBlock(OffsetDstIPv6a, mask[0], IPstack[0] , CMP_EQ, FUNC_NONE, NULL )
				);
				break;
			case DIR_UNSPEC:
			case SOURCE_OR_DESTINATION:
				yyval.param.self = Connect_OR(
					Connect_AND(
						NewBlock(OffsetSrcIPv6b, mask[1], IPstack[1] , CMP_EQ, FUNC_NONE, NULL ),
						NewBlock(OffsetSrcIPv6a, mask[0], IPstack[0] , CMP_EQ, FUNC_NONE, NULL )
					),
					Connect_AND(
						NewBlock(OffsetDstIPv6b, mask[1], IPstack[1] , CMP_EQ, FUNC_NONE, NULL ),
						NewBlock(OffsetDstIPv6a, mask[0], IPstack[0] , CMP_EQ, FUNC_NONE, NULL )
					)
				);		
				break;
			case SOURCE_AND_DESTINATION:
				yyval.param.self = Connect_AND(
					Connect_AND(
						NewBlock(OffsetSrcIPv6b, mask[1], IPstack[1] , CMP_EQ, FUNC_NONE, NULL ),
						NewBlock(OffsetSrcIPv6a, mask[0], IPstack[0] , CMP_EQ, FUNC_NONE, NULL )
					),
					Connect_AND(
						NewBlock(OffsetDstIPv6b, mask[1], IPstack[1] , CMP_EQ, FUNC_NONE, NULL ),
						NewBlock(OffsetDstIPv6a, mask[0], IPstack[0] , CMP_EQ, FUNC_NONE, NULL )
					)
				);
				break;
			default:
				/* should never happen */
				yyerror("This token is not expected here!");
				YYABORT;
		} /* End of switch*/

	}
break;
case 72:
#line 1654 "grammar.y"
	{ 
		int af, bytes, ret;
		uint64_t	mask[2];

		ret = parse_ip(&af, yystack.l_mark[-2].s, IPstack, &bytes, STRICT_IP, &num_ip);
		if ( ret == 0 ) {
			yyerror("Invalid IP address");
			YYABORT;
		}
		if ( ret == -1 ) {
			yyerror("IP address required - hostname not allowed here.");
			YYABORT;
		}
		/* ret == -2 will never happen here, as STRICT_IP is set*/


		if ( yystack.l_mark[0].value > (bytes*8) ) {
			yyerror("Too many netbits for this IP address");
			YYABORT;
		}

		if ( af == PF_INET ) {
			mask[0] = 0xffffffffffffffffLL;
			mask[1] = 0xffffffffffffffffLL << ( 32 - yystack.l_mark[0].value );
		} else {	/* PF_INET6*/
			if ( yystack.l_mark[0].value > 64 ) {
				mask[0] = 0xffffffffffffffffLL;
				mask[1] = 0xffffffffffffffffLL << ( 128 - yystack.l_mark[0].value );
			} else {
				mask[0] = 0xffffffffffffffffLL << ( 64 - yystack.l_mark[0].value );
				mask[1] = 0;
			}
		}
		/* IP aadresses are stored in network representation */
		mask[0]	 = mask[0];
		mask[1]	 = mask[1];

		IPstack[0] &= mask[0];
		IPstack[1] &= mask[1];

		switch ( yystack.l_mark[-4].param.direction ) {
			case SOURCE:
				yyval.param.self = Connect_AND(
					NewBlock(OffsetSrcIPv6b, mask[1], IPstack[1] , CMP_EQ, FUNC_NONE, NULL ),
					NewBlock(OffsetSrcIPv6a, mask[0], IPstack[0] , CMP_EQ, FUNC_NONE, NULL )
				);
				break;
			case DESTINATION:
				yyval.param.self = Connect_AND(
					NewBlock(OffsetDstIPv6b, mask[1], IPstack[1] , CMP_EQ, FUNC_NONE, NULL ),
					NewBlock(OffsetDstIPv6a, mask[0], IPstack[0] , CMP_EQ, FUNC_NONE, NULL )
				);
				break;
			case DIR_UNSPEC:
			case SOURCE_OR_DESTINATION:
				yyval.param.self = Connect_OR(
					Connect_AND(
						NewBlock(OffsetSrcIPv6b, mask[1], IPstack[1] , CMP_EQ, FUNC_NONE, NULL ),
						NewBlock(OffsetSrcIPv6a, mask[0], IPstack[0] , CMP_EQ, FUNC_NONE, NULL )
					),
					Connect_AND(
						NewBlock(OffsetDstIPv6b, mask[1], IPstack[1] , CMP_EQ, FUNC_NONE, NULL ),
						NewBlock(OffsetDstIPv6a, mask[0], IPstack[0] , CMP_EQ, FUNC_NONE, NULL )
					)
				);
				break;
			case SOURCE_AND_DESTINATION:
				yyval.param.self = Connect_AND(
					Connect_AND(
						NewBlock(OffsetSrcIPv6b, mask[1], IPstack[1] , CMP_EQ, FUNC_NONE, NULL ),
						NewBlock(OffsetSrcIPv6a, mask[0], IPstack[0] , CMP_EQ, FUNC_NONE, NULL )
					),
					Connect_AND(
						NewBlock(OffsetDstIPv6b, mask[1], IPstack[1] , CMP_EQ, FUNC_NONE, NULL ),
						NewBlock(OffsetDstIPv6a, mask[0], IPstack[0] , CMP_EQ, FUNC_NONE, NULL )
					)
				);
				break;
			default:
				yyerror("This token is not expected here!");
				YYABORT;
		} /* End of switch*/

	}
break;
case 73:
#line 1739 "grammar.y"
	{
		if ( yystack.l_mark[0].value > 0xffffffffLL ) {
			yyerror("Input interface number must 0..2^32");
			YYABORT;
		}

		switch ( yystack.l_mark[-2].param.direction ) {
			case DIR_UNSPEC:
				yyval.param.self = Connect_OR(
					NewBlock(OffsetInOut, MaskInput, (yystack.l_mark[0].value << ShiftInput) & MaskInput, CMP_EQ, FUNC_NONE, NULL),
					NewBlock(OffsetInOut, MaskOutput, (yystack.l_mark[0].value << ShiftOutput) & MaskOutput, CMP_EQ, FUNC_NONE, NULL)
				);
				break;
			case DIR_IN: 
				yyval.param.self = NewBlock(OffsetInOut, MaskInput, (yystack.l_mark[0].value << ShiftInput) & MaskInput, CMP_EQ, FUNC_NONE, NULL); 
				break;
			case DIR_OUT: 
				yyval.param.self = NewBlock(OffsetInOut, MaskOutput, (yystack.l_mark[0].value << ShiftOutput) & MaskOutput, CMP_EQ, FUNC_NONE, NULL); 
				break;
			default:
				yyerror("This token is not expected here!");
				YYABORT;
		} /* End of switch*/

	}
break;
case 74:
#line 1765 "grammar.y"
	{	
		if ( yystack.l_mark[0].value > 65535 ) {
			yyerror("VLAN number of range 0..65535");
			YYABORT;
		}

		switch ( yystack.l_mark[-2].param.direction ) {
			case SOURCE:
				yyval.param.self = NewBlock(OffsetVlan, MaskSrcVlan, (yystack.l_mark[0].value << ShiftSrcVlan) & MaskSrcVlan, CMP_EQ, FUNC_NONE, NULL );
				break;
			case DESTINATION:
				yyval.param.self = NewBlock(OffsetVlan, MaskDstVlan, (yystack.l_mark[0].value << ShiftDstVlan) & MaskDstVlan, CMP_EQ, FUNC_NONE, NULL);
				break;
			case DIR_UNSPEC:
			case SOURCE_OR_DESTINATION:
				yyval.param.self = Connect_OR(
					NewBlock(OffsetVlan, MaskSrcVlan, (yystack.l_mark[0].value << ShiftSrcVlan) & MaskSrcVlan, CMP_EQ, FUNC_NONE, NULL ),
					NewBlock(OffsetVlan, MaskDstVlan, (yystack.l_mark[0].value << ShiftDstVlan) & MaskDstVlan, CMP_EQ, FUNC_NONE, NULL)
				);
				break;
			case SOURCE_AND_DESTINATION:
				yyval.param.self = Connect_AND(
					NewBlock(OffsetVlan, MaskSrcVlan, (yystack.l_mark[0].value << ShiftSrcVlan) & MaskSrcVlan, CMP_EQ, FUNC_NONE, NULL ),
					NewBlock(OffsetVlan, MaskDstVlan, (yystack.l_mark[0].value << ShiftDstVlan) & MaskDstVlan, CMP_EQ, FUNC_NONE, NULL)
				);
				break;
			default:
				yyerror("This token is not expected here!");
				YYABORT;
		} /* End of switch*/

	}
break;
case 75:
#line 1798 "grammar.y"
	{
		uint64_t	mac = VerifyMac(yystack.l_mark[0].s);
		if ( mac == 0 ) {
			yyerror("Invalid MAC address format");
			YYABORT;
		}
		switch ( yystack.l_mark[-2].param.direction ) {
			case DIR_UNSPEC: {
					uint32_t in, out;
					in  = Connect_OR(
						NewBlock(OffsetInSrcMAC, MaskMac, mac, CMP_EQ, FUNC_NONE, NULL ),
						NewBlock(OffsetInDstMAC, MaskMac, mac, CMP_EQ, FUNC_NONE, NULL )
					);
					out  = Connect_OR(
						NewBlock(OffsetOutSrcMAC, MaskMac, mac, CMP_EQ, FUNC_NONE, NULL ),
						NewBlock(OffsetOutDstMAC, MaskMac, mac, CMP_EQ, FUNC_NONE, NULL )
					);
					yyval.param.self = Connect_OR(in, out);
					} break;
			case DIR_IN:
					yyval.param.self = Connect_OR(
						NewBlock(OffsetInSrcMAC, MaskMac, mac, CMP_EQ, FUNC_NONE, NULL ),
						NewBlock(OffsetInDstMAC, MaskMac, mac, CMP_EQ, FUNC_NONE, NULL )
					);
					break;
			case DIR_OUT:
					yyval.param.self = Connect_OR(
						NewBlock(OffsetOutSrcMAC, MaskMac, mac, CMP_EQ, FUNC_NONE, NULL ),
						NewBlock(OffsetOutDstMAC, MaskMac, mac, CMP_EQ, FUNC_NONE, NULL )
					);
					break;
			case SOURCE:
					yyval.param.self = Connect_OR(
						NewBlock(OffsetInSrcMAC, MaskMac, mac, CMP_EQ, FUNC_NONE, NULL ),
						NewBlock(OffsetOutSrcMAC, MaskMac, mac, CMP_EQ, FUNC_NONE, NULL )
					);
					break;
			case DESTINATION:
					yyval.param.self = Connect_OR(
						NewBlock(OffsetInDstMAC, MaskMac, mac, CMP_EQ, FUNC_NONE, NULL ),
						NewBlock(OffsetOutDstMAC, MaskMac, mac, CMP_EQ, FUNC_NONE, NULL )
					);
					break;
			case IN_SRC: 
					yyval.param.self = NewBlock(OffsetInSrcMAC, MaskMac, mac, CMP_EQ, FUNC_NONE, NULL );
					break;
			case IN_DST: 
					yyval.param.self = NewBlock(OffsetInDstMAC, MaskMac, mac, CMP_EQ, FUNC_NONE, NULL );
					break;
			case OUT_SRC: 
					yyval.param.self = NewBlock(OffsetOutSrcMAC, MaskMac, mac, CMP_EQ, FUNC_NONE, NULL );
					break;
			case OUT_DST:
					yyval.param.self = NewBlock(OffsetOutDstMAC, MaskMac, mac, CMP_EQ, FUNC_NONE, NULL );
					break;
				break;
			default:
				yyerror("This token is not expected here!");
				YYABORT;
		} /* End of switch*/
	}
break;
case 76:
#line 1860 "grammar.y"
	{	
		if ( yystack.l_mark[0].value > MPLSMAX ) {
			yyerror("MPLS value out of range");
			YYABORT;
		}

		/* search for label1 - label10*/
		if ( strncasecmp(yystack.l_mark[-2].s, "label", 5) == 0 ) {
			uint64_t mask;
			uint32_t offset, shift;
			char *s = &yystack.l_mark[-2].s[5];
			if ( *s == '\0' ) {
				yyerror("Missing label number");
				YYABORT;
			}
			int i = (int)strtol(s, (char **)NULL, 10);

			switch (i) {
				case 1:
					offset	= OffsetMPLS12;
					mask	= MaskMPLSlabelOdd;
					shift	= ShiftMPLSlabelOdd;
					break;
				case 2:
					offset	= OffsetMPLS12;
					mask	= MaskMPLSlabelEven;
					shift	= ShiftMPLSlabelEven;
					break;
				case 3:
					offset	= OffsetMPLS34;
					mask	= MaskMPLSlabelOdd;
					shift	= ShiftMPLSlabelOdd;
					break;
				case 4:
					offset	= OffsetMPLS34;
					mask	= MaskMPLSlabelEven;
					shift	= ShiftMPLSlabelEven;
					break;
				case 5:
					offset	= OffsetMPLS56;
					mask	= MaskMPLSlabelOdd;
					shift	= ShiftMPLSlabelOdd;
					break;
				case 6:
					offset	= OffsetMPLS56;
					mask	= MaskMPLSlabelEven;
					shift	= ShiftMPLSlabelEven;
					break;
				case 7:
					offset	= OffsetMPLS78;
					mask	= MaskMPLSlabelOdd;
					shift	= ShiftMPLSlabelOdd;
					break;
				case 8:
					offset	= OffsetMPLS78;
					mask	= MaskMPLSlabelEven;
					shift	= ShiftMPLSlabelEven;
					break;
				case 9:
					offset	= OffsetMPLS910;
					mask	= MaskMPLSlabelOdd;
					shift	= ShiftMPLSlabelOdd;
					break;
				case 10:
					offset	= OffsetMPLS910;
					mask	= MaskMPLSlabelEven;
					shift	= ShiftMPLSlabelEven;
					break;
				default: 
					yyerror("MPLS label out of range 1..10");
					YYABORT;
			}
			yyval.param.self = NewBlock(offset, mask, (yystack.l_mark[0].value << shift) & mask, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL );

		} else if ( strcasecmp(yystack.l_mark[-2].s, "eos") == 0 ) {
			/* match End of Stack label */
			yyval.param.self = NewBlock(0, AnyMask, yystack.l_mark[0].value << 4, yystack.l_mark[-1].param.comp, FUNC_MPLS_EOS, NULL );

		} else if ( strncasecmp(yystack.l_mark[-2].s, "exp", 3) == 0 ) {
			uint64_t mask;
			uint32_t offset, shift;
			char *s = &yystack.l_mark[-2].s[3];
			if ( *s == '\0' ) {
				yyerror("Missing label number");
				YYABORT;
			}
			int i = (int)strtol(s, (char **)NULL, 10);

			if ( yystack.l_mark[0].value > 7 ) {
				yyerror("MPLS exp value out of range");
				YYABORT;
			}

			switch (i) {
				case 1:
					offset	= OffsetMPLS12;
					mask	= MaskMPLSexpOdd;
					shift	= ShiftMPLSexpOdd;
					break;
				case 2:
					offset	= OffsetMPLS12;
					mask	= MaskMPLSexpEven;
					shift	= ShiftMPLSexpEven;
					break;
				case 3:
					offset	= OffsetMPLS34;
					mask	= MaskMPLSexpOdd;
					shift	= ShiftMPLSexpOdd;
					break;
				case 4:
					offset	= OffsetMPLS34;
					mask	= MaskMPLSexpEven;
					shift	= ShiftMPLSexpEven;
					break;
				case 5:
					offset	= OffsetMPLS56;
					mask	= MaskMPLSexpOdd;
					shift	= ShiftMPLSexpOdd;
					break;
				case 6:
					offset	= OffsetMPLS56;
					mask	= MaskMPLSexpEven;
					shift	= ShiftMPLSexpEven;
					break;
				case 7:
					offset	= OffsetMPLS78;
					mask	= MaskMPLSexpOdd;
					shift	= ShiftMPLSexpOdd;
					break;
				case 8:
					offset	= OffsetMPLS78;
					mask	= MaskMPLSexpEven;
					shift	= ShiftMPLSexpEven;
					break;
				case 9:
					offset	= OffsetMPLS910;
					mask	= MaskMPLSexpOdd;
					shift	= ShiftMPLSexpOdd;
					break;
				case 10:
					offset	= OffsetMPLS910;
					mask	= MaskMPLSexpEven;
					shift	= ShiftMPLSexpEven;
					break;
				default: 
					yyerror("MPLS label out of range 1..10");
					YYABORT;
			}
			yyval.param.self = NewBlock(offset, mask, yystack.l_mark[0].value << shift, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL );

		} else {
			yyerror("Unknown MPLS option");
			YYABORT;
		}
	}
break;
case 77:
#line 2015 "grammar.y"
	{	
		uint32_t *opt = malloc(sizeof(uint32_t));
		if ( yystack.l_mark[0].value > MPLSMAX ) {
			yyerror("MPLS value out of range");
			YYABORT;
		}
		if ( opt == NULL) {
			yyerror("malloc() error");
			YYABORT;
		}
		*opt = yystack.l_mark[0].value << 4;
		yyval.param.self = NewBlock(0, AnyMask, yystack.l_mark[0].value << 4, CMP_EQ, FUNC_MPLS_ANY, opt );

	}
break;
case 78:
#line 2029 "grammar.y"
	{
		if ( yystack.l_mark[0].value > 255 ) {
			yyerror("Forwarding status of range 0..255");
			YYABORT;
		}
		yyval.param.self = NewBlock(OffsetStatus, MaskStatus, (yystack.l_mark[0].value << ShiftStatus) & MaskStatus, CMP_EQ, FUNC_NONE, NULL);
	}
break;
case 79:
#line 2037 "grammar.y"
	{
		uint64_t id = Get_fwd_status_id(yystack.l_mark[0].s);
		if (id == 256 ) {
			yyerror("Unknown forwarding status");
			YYABORT;
		}

		yyval.param.self = NewBlock(OffsetStatus, MaskStatus, (id << ShiftStatus) & MaskStatus, CMP_EQ, FUNC_NONE, NULL);

	}
break;
case 80:
#line 2048 "grammar.y"
	{
		if ( yystack.l_mark[0].value > 2 ) {
			yyerror("Flow direction status of range 0, 1");
			YYABORT;
		}
		yyval.param.self = NewBlock(OffsetDir, MaskDir, (yystack.l_mark[0].value << ShiftDir) & MaskDir, CMP_EQ, FUNC_NONE, NULL);

	}
break;
case 81:
#line 2057 "grammar.y"
	{
		uint64_t dir = 0xFF;
		if ( yystack.l_mark[0].param.inout == INGRESS )
			dir = 0;
		else if ( yystack.l_mark[0].param.inout == EGRESS )
			dir = 1;
		else {
			yyerror("Flow direction status of range ingress, egress");
			YYABORT;
		}

		yyval.param.self = NewBlock(OffsetDir, MaskDir, (dir << ShiftDir) & MaskDir, CMP_EQ, FUNC_NONE, NULL);

	}
break;
case 82:
#line 2072 "grammar.y"
	{
		yyval.param.self = NewBlock(OffsetObservationDomainID, MaskObservationDomainID, (yystack.l_mark[0].value << ShiftObservationDomainID) & MaskObservationDomainID, CMP_EQ, FUNC_NONE, NULL);
	}
break;
case 83:
#line 2076 "grammar.y"
	{
		yyval.param.self = NewBlock(OffsetObservationPointID, MaskObservationPointID, (yystack.l_mark[0].value << ShiftObservationPointID) & MaskObservationPointID, CMP_EQ, FUNC_NONE, NULL);
	}
break;
case 84:
#line 2081 "grammar.y"
	{ 
		int i, af, bytes, ret;
		struct IPListNode *node;

		IPlist_t *root = malloc(sizeof(IPlist_t));

		if ( root == NULL) {
			yyerror("malloc() error");
			YYABORT;
		}
		RB_INIT(root);

		ret = parse_ip(&af, yystack.l_mark[0].s, IPstack, &bytes, ALLOW_LOOKUP, &num_ip);

		if ( ret == 0 ) {
			yyerror("Invalid IP address");
			YYABORT;
		}
		/* ret == -1 will never happen here, as ALLOW_LOOKUP is set*/
		
		if ( ret != -2 ) {
			if ( af && (( af == PF_INET && bytes != 4 ) || ( af == PF_INET6 && bytes != 16 ))) {
				yyerror("incomplete IP address");
				YYABORT;
			}

			for ( i=0; i<num_ip; i++ ) {
				if ((node = malloc(sizeof(struct IPListNode))) == NULL) {
					yyerror("malloc() error");
					YYABORT;
				}
				node->ip[0] = IPstack[2*i];
				node->ip[1] = IPstack[2*i+1];
				node->mask[0] = 0xffffffffffffffffLL;
				node->mask[1] = 0xffffffffffffffffLL;
				RB_INSERT(IPtree, root, node);
			}

		}
		yyval.list = (void *)root;

	}
break;
case 85:
#line 2124 "grammar.y"
	{ 
		int af, bytes, ret;
		struct IPListNode *node;

		IPlist_t *root = malloc(sizeof(IPlist_t));

		if ( root == NULL) {
			yyerror("malloc() error");
			YYABORT;
		}
		RB_INIT(root);

		ret = parse_ip(&af, yystack.l_mark[-2].s, IPstack, &bytes, STRICT_IP, &num_ip);

		if ( ret == 0 ) {
			yyerror("Invalid IP address");
			YYABORT;
		}
		/* ret == -1 will never happen here, as ALLOW_LOOKUP is set*/
		
		if ( ret != -2 ) {
			if ( af && (( af == PF_INET && bytes != 4 ) || ( af == PF_INET6 && bytes != 16 ))) {
				yyerror("incomplete IP address");
				YYABORT;
			}

			if ((node = malloc(sizeof(struct IPListNode))) == NULL) {
				yyerror("malloc() error");
				YYABORT;
			}

			if ( af == PF_INET ) {
				node->mask[0] = 0xffffffffffffffffLL;
				node->mask[1] = 0xffffffffffffffffLL << ( 32 - yystack.l_mark[0].value );
			} else {	/* PF_INET6*/
				if ( yystack.l_mark[0].value > 64 ) {
					node->mask[0] = 0xffffffffffffffffLL;
					node->mask[1] = 0xffffffffffffffffLL << ( 128 - yystack.l_mark[0].value );
				} else {
					node->mask[0] = 0xffffffffffffffffLL << ( 64 - yystack.l_mark[0].value );
					node->mask[1] = 0;
				}
			}

			node->ip[0] = IPstack[0] & node->mask[0];
			node->ip[1] = IPstack[1] & node->mask[1];

			RB_INSERT(IPtree, root, node);

		}
		yyval.list = (void *)root;

	}
break;
case 86:
#line 2178 "grammar.y"
	{ 
		int i, af, bytes, ret;
		struct IPListNode *node;

		ret = parse_ip(&af, yystack.l_mark[0].s, IPstack, &bytes, ALLOW_LOOKUP, &num_ip);

		if ( ret == 0 ) {
			yyerror("Invalid IP address");
			YYABORT;
		}
		if ( af && (( af == PF_INET && bytes != 4 ) || ( af == PF_INET6 && bytes != 16 ))) {
			yyerror("incomplete IP address");
			YYABORT;
		}

		/* ret == - 2 means lookup failure*/
		if ( ret != -2 ) {
			for ( i=0; i<num_ip; i++ ) {
				if ((node = malloc(sizeof(struct IPListNode))) == NULL) {
					yyerror("malloc() error");
					YYABORT;
				}
				node->ip[0] = IPstack[2*i];
				node->ip[1] = IPstack[2*i+1];
				node->mask[0] = 0xffffffffffffffffLL;
				node->mask[1] = 0xffffffffffffffffLL;
	
				RB_INSERT(IPtree, (IPlist_t *)yyval.list, node);
			}
		}
	}
break;
case 87:
#line 2209 "grammar.y"
	{ 
		int i, af, bytes, ret;
		struct IPListNode *node;

		ret = parse_ip(&af, yystack.l_mark[0].s, IPstack, &bytes, ALLOW_LOOKUP, &num_ip);

		if ( ret == 0 ) {
			yyerror("Invalid IP address");
			YYABORT;
		}
		if ( af && (( af == PF_INET && bytes != 4 ) || ( af == PF_INET6 && bytes != 16 ))) {
			yyerror("incomplete IP address");
			YYABORT;
		}

		/* ret == - 2 means lookup failure*/
		if ( ret != -2 ) {
			for ( i=0; i<num_ip; i++ ) {
				if ((node = malloc(sizeof(struct IPListNode))) == NULL) {
					yyerror("malloc() error");
					YYABORT;
				}
				node->ip[0] = IPstack[2*i];
				node->ip[1] = IPstack[2*i+1];
				node->mask[0] = 0xffffffffffffffffLL;
				node->mask[1] = 0xffffffffffffffffLL;
	
				RB_INSERT(IPtree, (IPlist_t *)yyval.list, node);
			}
		}
	}
break;
case 88:
#line 2241 "grammar.y"
	{ 
		int af, bytes, ret;
		struct IPListNode *node;

		ret = parse_ip(&af, yystack.l_mark[-2].s, IPstack, &bytes, STRICT_IP, &num_ip);

		if ( ret == 0 ) {
			yyerror("Invalid IP address");
			YYABORT;
		}
		if ( af && (( af == PF_INET && bytes != 4 ) || ( af == PF_INET6 && bytes != 16 ))) {
			yyerror("incomplete IP address");
			YYABORT;
		}

		/* ret == - 2 means lookup failure*/
		if ( ret != -2 ) {
			if ((node = malloc(sizeof(struct IPListNode))) == NULL) {
				yyerror("malloc() error");
				YYABORT;
			}
			if ( af == PF_INET ) {
				node->mask[0] = 0xffffffffffffffffLL;
				node->mask[1] = 0xffffffffffffffffLL << ( 32 - yystack.l_mark[0].value );
			} else {	/* PF_INET6*/
				if ( yystack.l_mark[0].value > 64 ) {
					node->mask[0] = 0xffffffffffffffffLL;
					node->mask[1] = 0xffffffffffffffffLL << ( 128 - yystack.l_mark[0].value );
				} else {
					node->mask[0] = 0xffffffffffffffffLL << ( 64 - yystack.l_mark[0].value );
					node->mask[1] = 0;
				}
			}

			node->ip[0] = IPstack[0] & node->mask[0];
			node->ip[1] = IPstack[1] & node->mask[1];

			RB_INSERT(IPtree, (IPlist_t *)yyval.list, node);
		}
	}
break;
case 89:
#line 2285 "grammar.y"
	{ 
		struct ULongListNode *node;

		ULongtree_t *root = malloc(sizeof(ULongtree_t));

		if ( root == NULL) {
			yyerror("malloc() error");
			YYABORT;
		}
		RB_INIT(root);

		if ((node = malloc(sizeof(struct ULongListNode))) == NULL) {
			yyerror("malloc() error");
			YYABORT;
		}
		node->value = yystack.l_mark[0].value;

		RB_INSERT(ULongtree, root, node);
		yyval.list = (void *)root;
	}
break;
case 90:
#line 2306 "grammar.y"
	{ 
		struct ULongListNode *node;

		if ((node = malloc(sizeof(struct ULongListNode))) == NULL) {
			yyerror("malloc() error");
			YYABORT;
		}
		node->value = yystack.l_mark[0].value;
		RB_INSERT(ULongtree, (ULongtree_t *)yyval.list, node);
	}
break;
case 91:
#line 2317 "grammar.y"
	{ 
		struct ULongListNode *node;

		if ((node = malloc(sizeof(struct ULongListNode))) == NULL) {
			yyerror("malloc() error");
			YYABORT;
		}
		node->value = yystack.l_mark[0].value;
		RB_INSERT(ULongtree, (ULongtree_t *)yyval.list, node);
	}
break;
case 92:
#line 2331 "grammar.y"
	{ yyval.param.comp = CMP_EQ; }
break;
case 93:
#line 2332 "grammar.y"
	{ yyval.param.comp = CMP_EQ; }
break;
case 94:
#line 2333 "grammar.y"
	{ yyval.param.comp = CMP_LT; }
break;
case 95:
#line 2334 "grammar.y"
	{ yyval.param.comp = CMP_GT; }
break;
case 96:
#line 2335 "grammar.y"
	{ yyval.param.comp = CMP_LE; }
break;
case 97:
#line 2336 "grammar.y"
	{ yyval.param.comp = CMP_GE; }
break;
case 98:
#line 2340 "grammar.y"
	{ yyval.param.direction = DIR_UNSPEC;  			 }
break;
case 99:
#line 2341 "grammar.y"
	{ yyval.param.direction = SOURCE;				 }
break;
case 100:
#line 2342 "grammar.y"
	{ yyval.param.direction = DESTINATION;			 }
break;
case 101:
#line 2343 "grammar.y"
	{ yyval.param.direction = SOURCE_OR_DESTINATION;  }
break;
case 102:
#line 2344 "grammar.y"
	{ yyval.param.direction = SOURCE_OR_DESTINATION;  }
break;
case 103:
#line 2345 "grammar.y"
	{ yyval.param.direction = SOURCE_AND_DESTINATION; }
break;
case 104:
#line 2346 "grammar.y"
	{ yyval.param.direction = SOURCE_AND_DESTINATION; }
break;
case 105:
#line 2347 "grammar.y"
	{ yyval.param.direction = DIR_IN;				 }
break;
case 106:
#line 2348 "grammar.y"
	{ yyval.param.direction = DIR_OUT;				 }
break;
case 107:
#line 2349 "grammar.y"
	{ yyval.param.direction = IN_SRC;				 }
break;
case 108:
#line 2350 "grammar.y"
	{ yyval.param.direction = IN_DST;				 }
break;
case 109:
#line 2351 "grammar.y"
	{ yyval.param.direction = OUT_SRC;				 }
break;
case 110:
#line 2352 "grammar.y"
	{ yyval.param.direction = OUT_DST;				 }
break;
case 111:
#line 2353 "grammar.y"
	{ yyval.param.direction = ADJ_PREV;				 }
break;
case 112:
#line 2354 "grammar.y"
	{ yyval.param.direction = ADJ_NEXT;				 }
break;
case 113:
#line 2357 "grammar.y"
	{ yyval.param.inout	= INGRESS;	}
break;
case 114:
#line 2358 "grammar.y"
	{ yyval.param.inout	= EGRESS;   }
break;
case 115:
#line 2361 "grammar.y"
	{ yyval.param.acl = ACL; 	}
break;
case 116:
#line 2362 "grammar.y"
	{ yyval.param.acl = ACE;		}
break;
case 117:
#line 2363 "grammar.y"
	{ yyval.param.acl = XACE;	}
break;
case 118:
#line 2366 "grammar.y"
	{ yyval.value = yystack.l_mark[0].param.self;        }
break;
case 119:
#line 2367 "grammar.y"
	{ yyval.value = Connect_OR(yystack.l_mark[-2].value, yystack.l_mark[0].value);  }
break;
case 120:
#line 2368 "grammar.y"
	{ yyval.value = Connect_AND(yystack.l_mark[-2].value, yystack.l_mark[0].value); }
break;
case 121:
#line 2369 "grammar.y"
	{ yyval.value = Invert(yystack.l_mark[0].value);			}
break;
case 122:
#line 2370 "grammar.y"
	{ yyval.value = yystack.l_mark[-1].value; }
break;
case 123:
#line 2371 "grammar.y"
	{ 
		yyval.value = yystack.l_mark[-3].value; 
		if ( strlen(yystack.l_mark[0].s) > 16 ) {
			yyerror("Error: Maximum 16 chars allowed for flowlabel");
			YYABORT;
		} else {
			AddLabel(yystack.l_mark[-3].value, yystack.l_mark[0].s);
		}
	}
break;
case 124:
#line 2380 "grammar.y"
	{ 
		yyval.value = yystack.l_mark[-1].value; 
		if ( strlen(yystack.l_mark[-3].s) > 16 ) {
			yyerror("Error: Maximum 16 chars allowed for flowlabel");
			YYABORT;
		} else {
			AddLabel(yystack.l_mark[-1].value, yystack.l_mark[-3].s);
		}
	}
break;
#line 3558 "grammar.c"
    }
    yystack.s_mark -= yym;
    yystate = *yystack.s_mark;
    yystack.l_mark -= yym;
    yym = yylhs[yyn];
    if (yystate == 0 && yym == 0)
    {
#if YYDEBUG
        if (yydebug)
            printf("%sdebug: after reduction, shifting from state 0 to\
 state %d\n", YYPREFIX, YYFINAL);
#endif
        yystate = YYFINAL;
        *++yystack.s_mark = YYFINAL;
        *++yystack.l_mark = yyval;
        if (yychar < 0)
        {
            yychar = YYLEX;
            if (yychar < 0) yychar = YYEOF;
#if YYDEBUG
            if (yydebug)
            {
                if ((yys = yyname[YYTRANSLATE(yychar)]) == NULL) yys = yyname[YYUNDFTOKEN];
                printf("%sdebug: state %d, reading %d (%s)\n",
                        YYPREFIX, YYFINAL, yychar, yys);
            }
#endif
        }
        if (yychar == YYEOF) goto yyaccept;
        goto yyloop;
    }
    if (((yyn = yygindex[yym]) != 0) && (yyn += yystate) >= 0 &&
            yyn <= YYTABLESIZE && yycheck[yyn] == (YYINT) yystate)
        yystate = yytable[yyn];
    else
        yystate = yydgoto[yym];
#if YYDEBUG
    if (yydebug)
        printf("%sdebug: after reduction, shifting from state %d \
to state %d\n", YYPREFIX, *yystack.s_mark, yystate);
#endif
    if (yystack.s_mark >= yystack.s_last && yygrowstack(&yystack) == YYENOMEM) goto yyoverflow;
    *++yystack.s_mark = (YYINT) yystate;
    *++yystack.l_mark = yyval;
    goto yyloop;

yyoverflow:
    YYERROR_CALL("yacc stack overflow");

yyabort:
    yyfreestack(&yystack);
    return (1);

yyaccept:
    yyfreestack(&yystack);
    return (0);
}
